#ifndef METHODTABLE_H
#define METHODTABLE_H

#include <ivrs/wrapperstl.h>

IVRS_NAMESPACE_BEGIN

class MethodInfo;
class MethodItr;


class MethodTable {
public:		

	enum MethodType { METHOD, CTOR, STATIC };

	// baseTable is copied into current table
	MethodTable(MethodTable* baseTable = 0);	

	// returns iterator of methods that fit  
	MethodItr query(const char* methname, int numargs, int methodtype = METHOD);
	
	// return all methods of type 'methodtype'
	const MethodVector* methods(int methodtype = METHOD);

	// insert method information	
	void insert(const char* methname, Wrapper::FKT wrapperfkt, const char* result,
				const char* args, const char* argnames, const char* defargs, 
				unsigned char minargs, unsigned char maxargs, int methodtype,
                const char* description = "");

	// update the internal counter of method 'methodname' to 'counter'
	void update(int id, int counter);

private:
	MethodVector mi_[3];
};



class MethodItr {
public:
	MethodItr(MethodVector* vec, const char* methname, int numargs) ;
    ~MethodItr();
	MethodInfo* operator*();
	void operator++() { pos_++; }
	void OK();
	void notOK();

	static int lookups;
	static int successfulCalls;
	static int failedCalls;

private:
	MethodVector* vec_;
	int pos_;
	const char* name_;
	int numargs_;
};


///////////////////////////////////////
// help struct
///////////////////////////////////////
class MethodInfo {
public:
	MethodInfo(MethodInfo* mi);
	MethodInfo(const char* name, Wrapper::FKT fkt, const char* result,
			 const char* args, const char* argnames, const char* defargs, 
			 unsigned char minargs, unsigned char maxargs, const char* description); 			 	

	const char* name_;
	Wrapper::FKT fkt_;
	const char* result_;
	const char* args_;
	const char* argnames_;
	const char* defargs_;
	unsigned char minargs_;
	unsigned char maxargs_;
    const char* description_;
	int counter_;	    
};	




IVRS_NAMESPACE_END

#endif // METHODTABLE_H
