#ifndef STRINGCONVERT_H
#define STRINGCONVERT_H

#include <ivrs/wrappermanager.h>
#include <vrs/sharedobj.h>
#include <vrs/vector.h>
#include <vrs/color.h>
#include <vrs/matrix.h>
#include <vrs/area.h>
#include <vrs/ray.h>
#include <vrs/bounds.h>
#include <string>

IVRS_NAMESPACE_BEGIN	
using namespace LDX;
using namespace VRS;

static const char* const NULLSTRING = "NULL";

// read functions for build in types
long			str2type(const char* in, long* dummy);
int				str2type(const char* in, int* dummy);
char			str2type(const char* in, char* dummy);
float			str2type(const char* in, float* dummy);
double			str2type(const char* in, double* dummy);
unsigned short	str2type(const char* in, unsigned short* dummy);
unsigned int	str2type(const char* in, unsigned int* dummy);
unsigned long	str2type(const char* in, unsigned long* dummy);
unsigned char	str2type(const char* in, unsigned char* dummy);
bool			str2type(const char* in, bool* out);
char*			str2type(const char* in, char** out);
std::string		str2type(const char* in, std::string* out);
std::wstring	str2type(const char* in, std::wstring* out);

// read functions for call-by-value types
Matrix    str2type(const char* in, Matrix*);
Vector    str2type(const char* in, Vector*);
Color     str2type(const char* in, Color*);
Area      str2type(const char* in, Area*);
Bounds    str2type(const char* in, Bounds*);
Ray       str2type(const char* in, Ray*);
const ID& str2type(const char* in, ID*);

// SO<> read function
template<class T>
inline SO<T> str2type(const char* in, SO<T>*) { return str2type(in,(T**)0); }

template<class T>
T* str2type(const char* in, T**) { 
    if (!strcmp(in,NULLSTRING)) { return 0; }
	Wrapper* wrp = WrapperManager::find(in,&typeid(T));
	if (!wrp) throw ConvertException(typeid(T).name(),in);
	return reinterpret_cast<T*>(wrp->obj());
}

//////////////////////////////////////////////////////
// type2str
//////////////////////////////////////////////////////

// converter for build in types
const char* type2str(const long& t);
const char* type2str(const int& t);
const char* type2str(const char& t);
const char* type2str(const float& t);
const char* type2str(const double& t);
const char* type2str(const bool& t);
const char* type2str(const unsigned short& t);
const char* type2str(const unsigned int& t);
const char* type2str(const unsigned long& t);
const char* type2str(const unsigned char& t);
const char* type2str(const char*& t);
const char* type2str(const void* obj);
const char* type2str(const SharedObj* obj);
const char* type2str(const std::string& t);
const char* type2str(const std::wstring& t);

// converter for call-by-value types
const char* type2str(const Matrix& obj);
const char* type2str(const Vector& obj);
const char* type2str(const Color& obj);
const char* type2str(const Area& obj);
const char* type2str(const Bounds& obj);
const char* type2str(const Ray& obj);
const char* type2str(const ID& obj);

// spezialized type2str Iterator template
template <class T>
inline const char* type2str(const SO<T>& i) {
	return type2str((T*)i);	
}

// specialized converter for OpenGL defines
const char* type2strGL(const int& t);
const char* type2strGL(const unsigned int& t);
int          str2typeGL(const char* in, int*);
unsigned int str2typeGL(const char* in, unsigned int*);

// string cleanup 
void deleteString(char* ptr); 

IVRS_NAMESPACE_END

#endif // STRINGCONVERT_H
