/******************************************************************************
 * VRS - The Virtual Rendering System
 * Copyright (C) 2003 Computer Graphics Systems Group at the 
 * Hasso-Plattner-Institute (HPI), Potsdam, Germany.
 *
 * This  library  is  free  software; you can redistribute it and/or  modify it
 * under the terms of the GNU Lesser General Public  License  as  published  by
 * the  Free  Software  Foundation;  either  version 2.1 of the License, or (at
 * your option) any later version. This library  is  distributed  in  the  hope
 * that  it will be useful, but  WITHOUT ANY WARRANTY; without even the implied
 * warranty of  MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE.  See  the
 * GNU Lesser General Public License for more details. You should have received
 * a copy of the GNU Lesser+ General Public License along with this library; if
 * not, write to the FreeSoftware Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA, 02111-1307, USA.
 ******************************************************************************/

// $Id: tcltemplatecallback.h,v 1.5 2004/07/04 15:20:02 kirsch Exp $
// $Date: 2004/07/04 15:20:02 $
// $Revision: 1.5 $
// $State: Exp $
// $Author: kirsch $
//
// $Log: tcltemplatecallback.h,v $
// Revision 1.5  2004/07/04 15:20:02  kirsch
// iVRS updates
//
// Revision 1.4  2003/02/19 14:20:48  baumann
// - code cleanup
// - TclCallbackR2<...> added
//
// Revision 1.3  2002/05/08 12:47:28  kersting
// added persistency
//
// Revision 1.2  2002/04/02 07:59:24  kersting
// using RTTI for type recognition
//
// Revision 1.1  2002/02/27 15:09:06  kersting
// initial version
//

#ifndef IVRS_TCLTEMPLATECALLBACK_H
#define IVRS_TCLTEMPLATECALLBACK_H

#include <vrs/callback.h>

namespace VRS {

template<class RETURN>
class TclCallbackR : public VRS::CallbackR<RETURN> {
public:
    TclCallbackR(const std::string& script = "") : script_(script) { }

    virtual RETURN callback();
    
    const std::string& getScript() const { return script_; }
    void setScript(const std::string& script) { script_ = script; modified(); }

    VRS_TYPEINFO(TclCallbackR, VRS::CallbackR<RETURN>);
    VRS_SERIALIZABLE(TclCallbackR);

private:
    std::string script_;
};

template<class ARG1>
class TclCallback1 : public VRS::Callback1<ARG1> {
public:
    TclCallback1(const std::string& script = "") : script_(script) { }

    virtual void callback(ARG1 arg1);

    const std::string& getScript() const { return script_; }
    void setScript(const std::string& script) { script_ = script; modified(); }

    VRS_TYPEINFO(TclCallback1, VRS::Callback1<ARG1>);
    VRS_SERIALIZABLE(TclCallback1);

private:
    std::string script_;
};

template<class RETURN, class ARG1>
class TclCallbackR1 : public VRS::CallbackR1<RETURN, ARG1> {
public:
    TclCallbackR1(const std::string& script = "") : script_(script) { }

    virtual RETURN callback(ARG1 arg1);

    const std::string& getScript() const { return script_; }
    void setScript(const std::string& script) { script_ = script; modified(); }
    
    VRS_TYPEINFO(TclCallbackR1, VRS_TEMPLATE_ARGS_2(RETURN, ARG1, VRS::CallbackR1));
    VRS_SERIALIZABLE(TclCallbackR1);

private: 
    std::string script_;
};

template<class RETURN, class ARG1, class ARG2>
class TclCallbackR2 : public VRS::CallbackR2<RETURN, ARG1, ARG2> {
public:
    TclCallbackR2(const std::string& script = "") : script_(script) { }

    virtual RETURN callback(ARG1 arg1, ARG2 arg2);

    const std::string& getScript() const { return script_; }
    void setScript(const std::string& script) { script_ = script; modified(); }
    
    VRS_TYPEINFO(TclCallbackR2, VRS_TEMPLATE_ARGS_3(RETURN, ARG1, ARG2, VRS::CallbackR2));
    VRS_SERIALIZABLE(TclCallbackR2);

private: 
    std::string script_;
};

} // namespace VRS

#include <ivrs/tcl/tcltemplatecallback.tli>

#endif // IVRS_TCLTEMPLATECALLBACK_H
