#ifndef WRAPPER_H
#define WRAPPER_H

#include <ivrs/wrapperexception.h>
#include <ivrs/wrapperclassinfo.h>

IVRS_NAMESPACE_BEGIN

class MethodTable;
class EnumTable;

class Wrapper {
public:	
	
	typedef char* (Wrapper::*FKT)(int argc, const char** argv);	

	Wrapper(void* masterobj = 0);
	virtual ~Wrapper();

	// initialize wrapper subobj
	void initializeObject(int argc, const char** argv);	

	// call method of wrapper subobj
	const char* callObjectMethod(int argc, const char** argv);
	const char* callStaticMethod(int argc, const char** argv);

    // destroy wrapper subobject
    virtual void destroyObject();

    // returns table of wrapped functions
	virtual MethodTable* methodTable() const;

	// returns table of class enums
	virtual EnumTable* enumtable() const;
    
    // clone method
    virtual Wrapper* clone(void* masterobj) const;        

	// give access to the masterobj
	void* obj() const;	

	// name of the wrapper
	char* name() const;	
    
    // returns method info to given methodname
    const char* methodInfo(int argc,const  char** argv);

    static WrapperClassInfo* WrapperInfo();
    virtual WrapperClassInfo* wrapperInfo() const;
    bool isDerivedFrom(WrapperClassInfo* info) const;

protected: 
	void* obj_;
	char* name_;
};

IVRS_NAMESPACE_END


#endif // WRAPPER_H
