#include <ivrs/enumtable.h>
#include <ivrs/wrapperexception.h>
#include <ivrs/wrappermanager.h>
#include <strstream>

IVRS_NAMESPACE_BEGIN


EnumTable::EnumTable(WrapperClassInfo* scopename) {
    scopename_ = scopename  ? scopename->masterClassName() : 0; 
}

void EnumTable::registerEnum(const char* enumname, const char* enumtype, int value) {
	Enumname2Int* map = 0;
	ETI vi = vec_.find(enumtype);
	if (vi == vec_.end()) {
		map = new Enumname2Int;
		vec_.insert(Enumtype2Map::value_type(enumtype,map));
	} else {
		map = vi->second;
	}
	map->insert(Enumname2Int::value_type(enumname,value));
}

int EnumTable::enumValue(const char* fullname, const char* enumtype) {

	const char* enumname = fullname;	
	if (scopename_ && strrchr(enumname,':')) {
		int slen = strlen(scopename_);
		if (slen+2 > strlen(fullname) || strncmp(fullname,scopename_,slen)) {
			throw InternalException("no enum value found for enumname ",fullname,0);	
		}
		enumname = fullname + slen + 2;
	}

	for (ETI vi = vec_.begin(); vi != vec_.end(); vi++) {
		if (!enumtype || !strcmp(enumtype,vi->first)) {
			Enumname2Int* map = vi->second;
			EII ei = map->find(enumname);
			if (ei != map->end()) {
				return ei->second;
			}
			if (enumtype) {	
				throw InternalException("no enumname ",fullname,0); 
			}
		}
	}

	throw InternalException("no enum value found for enumname ",fullname,0);	
	return 0;
}

const char* EnumTable::enumName(int value, const char* enumtype) {
	
	for (ETI vi = vec_.begin(); vi != vec_.end(); vi++) {
		if (!enumtype || !strcmp(enumtype,vi->first)) {
			Enumname2Int* map = vi->second;
			for (EII ei = map->begin(); ei != map->end(); ei++) {
				if (ei->second == value) { 
                    char* tmp;
                    if (scopename_) {
					    tmp = new char[strlen(ei->first)+strlen(scopename_)+3];					                    
					    sprintf(tmp,"%s::%s",scopename_,ei->first);
                    } else {
                        tmp = new char[strlen(ei->first)+1];
                        strcpy(tmp,ei->first);
                    }
					return tmp;
				}
			}
			if (enumtype) {	throw InternalException("no enumname found ",enumtype,0); }
		}
	}

	throw InternalException("no enumname found ",enumtype,0);	
	return 0;
}    

const char* EnumTable::allEnums(const char* enumtype) {	
	std::strstream buf;
    
	for (ETI vi = vec_.begin(); vi != vec_.end(); vi++) {
		Enumname2Int* map = vi->second;
        if (!enumtype || !strcmp(enumtype,vi->first)) {
		    for (EII ei = map->begin(); ei != map->end(); ei++) {
    			buf << ei->first << " ";
		    }
        }
	}     
    buf << std::ends;	
	return buf.str();
}    


int EnumTable::scopeEnumValue(const char* scope_enumname) {
	// name syntax is classname::enumname for class enumerations
	// and enumname for global defines (OpenGL)
	static char name[64];
	strcpy(name,scope_enumname);

	char* f = strchr(name,':');
	char* l = strrchr(name,':');
	char* scopename = 0;
	char* enumname = 0;

	if (l == f) {
		scopename = "GL";
		enumname = name;
	} else {
		f[0] = '\0';
		scopename = name;
		enumname = l + 1;
	}
	
	Wrapper* wrp = WrapperManager::wrapperPrototype(scopename);
	if (!wrp) { throw InternalException("no scope found for enum ",scope_enumname,0); }

	EnumTable* etb = wrp->enumtable();
	if (!etb) { throw InternalException("no enumtable found for scope ",scopename,0); }
	
	return etb->enumValue(enumname);
}

IVRS_NAMESPACE_END