#include <vrs/color.h>
#include <vrs/vector.h>
#include <vrs/matrix.h>
#include <vrs/bounds.h>
#include <vrs/ray.h>
#include <vrs/area.h>

#include <ivrs/tcl/tclcallmanager.h>
#include <ivrs/wrappermanager.h>
#include <ivrs/enumtable.h>
#include <ivrs/stringconvert.h>
#include <ivrs/gen/idwrp.h>

#include <vrs/lookat.h>
#include <strstream>
#include <strstream>

IVRS_NAMESPACE_BEGIN

static char dummy;

int str2type(const char* in, int*) {
	int v;
	if (sscanf(in,"%i",&v) != 1) {
		try {
			static char buf[64];
			const char* pos = in;
			const char* ori = in;
			int value = 0;
			while (ori) {
				ori = strpbrk(pos,"|");
				if (ori) {
					strncpy(buf,pos,(ori-pos));
					buf[(ori-pos)] = '\0';
					value |= EnumTable::scopeEnumValue(buf);
					pos = ori+1;
				} else {
					strcpy(buf,pos);
					value |= EnumTable::scopeEnumValue(buf);
				}
			}
			return value;
		} catch (InternalException) {
			throw ConvertException("int",in);	
		}
	}
	return v;
}

long str2type(const char* in, long*) {
	long v;
	if (sscanf(in,"%li",&v) != 1) {
		throw ConvertException("long",in);	
	}
	return v;
}
char str2type(const char* in, char*) {
	char v;
	if (sscanf(in,"%1s",&v) != 1) {
		throw ConvertException("char",in);	
	}
	return v;
}
float str2type(const char* in, float*) {
	float v;
	if (sscanf(in,"%f %c",&v,&dummy) != 1) {
		throw ConvertException("float",in);	
	}
	return v;
}
double str2type(const char* in, double*) {
	double v;	
	if (sscanf(in,"%lf %c",&v,&dummy) != 1) {
		throw ConvertException("double",in);	
	}
	return v;
}
unsigned short str2type(const char* in, unsigned short*) {
	unsigned int v;
	if (sscanf(in,"%u",&v) != 1) {
		throw ConvertException("unsigned short",in);	
	}
	return v;
}

unsigned int str2type(const char* in, unsigned int*) {
	unsigned int v;
	if (sscanf(in,"%u",&v) != 1) {
		throw ConvertException("unsigned int",in);	
	}
	return v;
}

unsigned char str2type(const char* in, unsigned char*) {
	unsigned char v;
	if (sscanf(in,"%hc",&v) != 1) {
		throw ConvertException("unsigned char",in);	
	}
	return v;
}
unsigned long str2type(const char* in, unsigned long*) {
	unsigned long v;
	if (sscanf(in,"%lu",&v) != 1) {
		throw ConvertException("unsigned long",in);	
	}
	return v;
}


Matrix str2type(const char* c, Matrix*) {
	Matrix o;
	std::istrstream in(c);
	if (!(in >> o)) {
		Wrapper* wrp = WrapperManager::find(c,&typeid(Matrix));
		if (wrp) {
			o = *reinterpret_cast<Matrix*>(wrp->obj());
		} else {
			throw ConvertException("Matrix",c);			
		}		
	}
	return o;
}

Vector str2type(const char* c, Vector*) {
	Vector o;
	if (sscanf(c," %lf %lf %lf %c",&o[0],&o[1],&o[2],&dummy) != 3) {
		Wrapper* wrp = WrapperManager::find(c,&typeid(Vector));
		if (wrp) {
			o = *reinterpret_cast<Vector*>(wrp->obj());
		} else {
			throw ConvertException("Vector",c);			
		}
	}	
	return o;
}

Color str2type(const char* in, Color*) {
	Color v;
	if (sscanf(in," %lf %lf %lf %lf %c",&v[0],&v[1],&v[2],&v[3],&dummy) != 4) {
		Wrapper* wrp = WrapperManager::find(in,&typeid(Color));
		if (wrp) {
			return *reinterpret_cast<Color*>(wrp->obj());
		} else {
			throw ConvertException("Color",in);			
		}
	}	
	return v;
}

Area str2type(const char* in, Area*) {	
	int x,y,w,h;	
	if (sscanf(in,"%i %i %i %i",&x,&y,&w,&h) != 4) {
		Wrapper* wrp = WrapperManager::find(in,&typeid(Area));
		if (wrp) {
			return *reinterpret_cast<Area*>(wrp->obj());
		} else {
			throw ConvertException("Area",in);			
		}
	}		
	return Area(x,y,w,h);
}


Bounds str2type(const char* in, Bounds*) {
	double x1,y1,z1,x2,y2,z2;
	if (sscanf(in," { %lf %lf %lf } { %lf %lf %lf } ",&x1,&y1,&z1,&x2,&y2,&z2) != 6) {
		Wrapper* wrp = WrapperManager::find(in,&typeid(Bounds));
		if (wrp) {
			return *reinterpret_cast<Bounds*>(wrp->obj());
		} else {
			throw ConvertException("Bounds",in);			
		}
	}
	
	return Bounds(Vector(x1,y1,z1),Vector(x2,y2,z2));
}

Ray str2type(const char* in, Ray*) {
	Vector p, d;	
	if (sscanf(in," %lf %lf %lf %lf %lf %lf ",&p[0],&p[1],&p[2],&d[0],&d[1],&d[2]) != 6) {	
		Wrapper* wrp = WrapperManager::find(in,&typeid(Ray));
		if (wrp) {
			return *reinterpret_cast<Ray*>(wrp->obj());
		} else {
			throw ConvertException("Ray",in);			
		}
	}	
	return Ray(p,d);
}

bool str2type(const char* in, bool*) {	
	if (!strcmp(in,"true") || !strcmp(in,"1")) { return true; }
	if (!strcmp(in,"false") || !strcmp(in,"0")) { return false; }
	throw ConvertException("bool",in);
}

std::string str2type(const char* in, std::string*) {
    return std::string(in);
}

std::wstring str2type(const char* in, std::wstring*) {
    if(in) {
        const unsigned int size = ::strlen(in);
        wchar_t* str = new wchar_t[size + 1];
        for(unsigned int i = 0; i < size; ++i) {
            str[i] = static_cast<wchar_t>(in[i]);
        }
        str[size] = static_cast<wchar_t>(0);
        const std::wstring result(str);
        delete [] str;
        return result;
    } else {
        return std::wstring();
    }
}

char* str2type(const char* in, char**) {
	return (char*)in;
}

const ID& str2type(const char* in, ID*) {
	Wrapper* wrp = WrapperManager::find(in,&typeid(ID));
	if (wrp) {
		return *reinterpret_cast<ID*>(wrp->obj());
	} 
	throw ConvertException("ID",in);
    static const ID empty;
    return empty;
}

const char* type2str(const ID& obj) {
	char* buf = new char[32];
	Wrapper* wrp = WrapperManager::find(&obj,&typeid(ID));
	if (!wrp) {			
		wrp = new Wrp_ID((void*)&obj);
		TclCallManager::createTclObject(wrp);			
	}
	strcpy(buf,wrp->name());

	return buf;
}

/////////////////////////////////////////////////
//  type2str
/////////////////////////////////////////////////

const char* type2str(const int& t) {
	char* buf = new char[16];
	sprintf(buf,"%i",t);	
	return buf;
}
const char* type2str(const long& t) {
	char* buf = new char[32];
	sprintf(buf,"%li",t);	
	return buf;
}
const char* type2str(const char& t) {
	char* buf = new char[6];
	sprintf(buf,"%c",t);	
	return buf;
}
const char* type2str(const float& t) {
	char* buf = new char[32];
	sprintf(buf,"%f",(double)t);	
	return buf;
}
const char* type2str(const double& t) {
	char* buf = new char[128];
	sprintf(buf,"%f",t);	
	return buf;
}
const char* type2str(const unsigned int& t) {
	char* buf = new char[32];
	sprintf(buf,"%u",t);	
	return buf;
}
const char* type2str(const unsigned short& t) {
	char* buf = new char[32];
	sprintf(buf,"%u",t);	
	return buf;
}
const char* type2str(const unsigned char& t) {
	char* buf = new char[32];
	sprintf(buf,"%i",(int)t);	
	return buf;
}
const char* type2str(const unsigned long& t) {
	char* buf = new char[32];
	sprintf(buf,"%li",t);	
	return buf;
}

const char* type2str(const Matrix& t) {
    std::ostrstream s;
    s << t << std::ends;
	return s.str();
}

const char* type2str(const Vector& t) {
	char* buf = new char[128];		
	sprintf(buf,"%f %f %f",t[0],t[1],t[2]);	
	return buf;
}

const char* type2str(const Color& t) {
	char* buf = new char[128];		
	sprintf(buf,"%f %f %f %f",t[0],t[1],t[2],t[3]);	
	return buf;
}

const char* type2str(const Bounds& t) {
	char* buf = new char[1024];	 	
    if (t.isDefined()) {
	    Vector llf = t.getLLF();
	    Vector urb = t.getURB();
	    sprintf(buf,"{ %f %f %f } { %f %f %f }",llf[0],llf[1],llf[2],urb[0],urb[1],urb[2]);	
    } else {
        strcpy(buf,"undefined");
    }
	return buf;
}

const char* type2str(const Area& r) {
	char* buf = new char[256];		
	int x = r.getX();
	int y = r.getY();
	int w = r.getWidth();
	int h = r.getHeight();
	sprintf(buf,"%i %i %i %i",x,y,w,h);	
	return buf;
}
    
const char* type2str(const Ray& t) {
	char* buf = new char[256];
	Vector o = t.getOrigin();
	Vector d = t.getDirection();
	sprintf(buf,"%f %f %f   %f %f %f",o[0],o[1],o[2],d[0],d[1],d[2]);	
	return buf;
}

const char* type2str(const std::string& t) {
	char* buf = new char[t.length()+1];
	strcpy(buf,t.c_str());
	return buf;
}

const char* type2str(const std::wstring& t) {
    const unsigned int size = t.length();
	char* buf = new char[size + 1];
    for(unsigned int i = 0; i < size; ++i) {
        buf[i] = static_cast<char>(t[i]);
    }
    buf[size] = 0;
	return buf;
}

const char* type2str(const bool& t) {
	char* buf = new char[2];
    buf[0] = t ? '1' : '0';
    buf[1] = 0;
	return buf;
}

const char* type2str(const const char*& t) {

	char* buf;
	if (t == 0) {
		buf = new char;
		buf[0] = '\0';
	} else {
		buf = new char[strlen(t)+1];
		strcpy(buf,t);
	}	
	return buf;	
}

const char* type2str(const void* obj) {	
	char* buf = new char[64];
	if (obj == 0) {	strcpy(buf,NULLSTRING);	} 
	else {		
		Wrapper* wrp = WrapperManager::find(obj,&typeid(void*));
		if (!wrp) {			
			wrp = new Wrapper((void*)obj);			
			TclCallManager::createTclObject(wrp);			
		}
		strcpy(buf,wrp->name());
	}
	return buf;
}

extern const const char* cleanRTTIName(const const char* name);

const char* type2str(const SharedObj* obj) {

    char* buf = new char[64];
	if (obj == 0) {
		strcpy(buf,NULLSTRING);
	} else {

        // look for already stored wrapper                        
        const ClassInfo* info = obj->classInfoVRS();
        VRS_Error(info,"no class info");
        
        Wrapper* wrp = WrapperManager::find((void*)obj,info->typeInfo());
		if (!wrp) {                        
            // SharedObj hierarchy is traversed to find valid wrapper
			const ClassInfo* info = obj->classInfoVRS();
            Wrapper* prt = 0;

			while (!prt && info) {                
                prt = WrapperManager::wrapperPrototype(info->typeInfo());                
				info = info->parent();
            }            

            // create new wrapper
            if (prt) { 
                wrp = prt->clone((void*)obj); 
            } else { 
                VRS_Warning(0,"no valid wrapper found, using 'void*' wrapper");
                wrp = new Wrapper((void*)obj); 
            } 

		    TclCallManager::createTclObject(wrp);	
		}        
		strcpy(buf,wrp->name());
	}
	return buf;
}

void deleteString(char* ptr) {
	delete[] ptr;
} 

IVRS_NAMESPACE_END
