#include <ivrs/wrapper.h>
#include <ivrs/methodtable.h>
#include <ivrs/wrappermanager.h>
#include <fstream>
#include <strstream>

IVRS_NAMESPACE_BEGIN	
using namespace LDX;
using namespace VRS;

Wrapper::Wrapper(void* obj) : obj_(obj), name_(0) {	
}

Wrapper::~Wrapper() {	
	delete[] name_;
}

void Wrapper::destroyObject() {
	obj_ = 0;
}

Wrapper* Wrapper::clone(void* obj) const {
	return new Wrapper(obj);
}

WrapperClassInfo* Wrapper::WrapperInfo() { 
        static WrapperClassInfo i( typeid(void*) , 0);
        return &i; 
} 
WrapperClassInfo* Wrapper::wrapperInfo() const { 
    return Wrapper::WrapperInfo(); 
}

bool Wrapper::isDerivedFrom(WrapperClassInfo* test) const {
    return true;
}

char* Wrapper::name() const { 	
	if (!name_) {
        char* clean = (char*)wrapperInfo()->masterClassName();
		char* name = new char[strlen(clean)+20];				
		sprintf(name,"obj%s%i",clean,(int)obj_);
		((Wrapper*)this)->name_ = name;
	}	
	return name_; 
}
 
void* Wrapper::obj() const { 
	return obj_; 
}	

void Wrapper::initializeObject(int argc, const char** argv) {
	MethodItr itr = methodTable()->query(0,argc,MethodTable::CTOR);
	
	MethodInfo* m = (*itr);
    char* lastmessage = 0;
	while (m) {		
		try {					
			(this->*(m->fkt_))(argc,argv);
			itr.OK(); // optimization
			return;
		} catch (ConvertException e) {
			itr.notOK();
			// argument convert error, try next
            std::cerr << (char*)e.message() << std::endl;
            lastmessage = (char*)e.message();
		} catch (Exception e) {		
            itr.notOK();
			// argument convert error, try next
            std::cerr << e.message() << std::endl;
            // lastmessage = e.reason();
        }
		++itr;
		m = (*itr);

	}

    if (lastmessage) { 
        throw InternalException(lastmessage,wrapperInfo()->masterClassName(),argv);  
    } else { 
        throw InternalException("no valid constructor found",wrapperInfo()->masterClassName(),argv);    
    }        
    
}



const char* Wrapper::callObjectMethod(int argc, const char** argv) {	
	MethodItr itr = methodTable()->query(argv[0],argc-1);		
	
	// iterate over all registered Wrapper methods
	// look for the methodsignature that fits		
	MethodInfo* m = (*itr);
    char* lastmessage = 0;
	while (m) {
		try {					
			char* result = (this->*(m->fkt_))(argc-1,argv+1);
			itr.OK(); // optimization
			return result;
		} catch (ConvertException& e) {
			itr.notOK();
			// argument convert error, try next
			//cerr << (char*)e.message() << endl;
            lastmessage = (char*)e.message();
		} 
		++itr;
		m = (*itr);
	} 

    if (lastmessage) { 
        throw InternalException(lastmessage,wrapperInfo()->masterClassName(),argv);    
    } else { 
        throw InternalException("object call: wrong arguments",wrapperInfo()->masterClassName(),argv);    
    }
	return 0;
}

const char* Wrapper::callStaticMethod(int argc, const char** argv) {		
	MethodItr itr = methodTable()->query(argv[0],argc-1,MethodTable::STATIC);
	
	// iterate over all registered Wrapper methods
	// look for the methodsignature that fits		
	MethodInfo* m = (*itr);
	while (m) {		
		try {					
			char* result = (this->*(m->fkt_))(argc-1,argv+1);
			itr.OK(); // optimization
			return result;
		} catch (ConvertException& e) {
			itr.notOK();
			// argument convert error, try next
			// cerr << (char*)e.message() << endl;
		} 
		++itr;
		m = (*itr);
	} 
    throw InternalException("static call: wrong arguments",wrapperInfo()->masterClassName(),argv);    
	return 0;

}
const char* Wrapper::methodInfo(int argc, const char** argv) {
    std::strstream buf;
	const MethodVector* table;
    
    table = methodTable()->methods(MethodTable::METHOD);
	for (int i = 0; i < table->size(); i++) {
		MethodInfo* mi = (*table)[i];
        if (argc == 0 || !strncmp(argv[0],mi->name_,strlen(argv[0]))) {
            buf << mi->result_ << " " << mi->name_ << " { " << mi->args_ << " } { " << mi->argnames_ << " } { " << mi->defargs_ << " } { " << mi->description_ << " }" << std::endl;		
        }
	}
    table = methodTable()->methods(MethodTable::CTOR);
    WrapperClassInfo* ci = wrapperInfo();
	for (int i = 0; i < table->size(); i++) {
		MethodInfo* mi = (*table)[i];
        if (argc == 0 || !strncmp(argv[0],mi->name_,strlen(argv[0]))) {
		    buf << ci->masterClassName() << " { " << mi->args_ << " } { " << mi->argnames_ << " } { " << mi->defargs_ << " } { " << mi->description_ << " }" << std::endl;		
        }
	}
    table = methodTable()->methods(MethodTable::STATIC);
	for (int i = 0; i < table->size(); i++) {
		MethodInfo* mi = (*table)[i];
        if (argc == 0 || !strncmp(argv[0],mi->name_,strlen(argv[0]))) {
            buf << mi->result_ << " " << mi->name_ << " { " << mi->args_ << " } { " << mi->argnames_ << " } { " << mi->defargs_ << " } { " << mi->description_ << " }" << std::endl;		
        }
	}
    buf << std::ends;
    return buf.str();
}

MethodTable* Wrapper::methodTable() const {
	static MethodTable* dta_ = new MethodTable();	
    dta_->insert("?",(Wrapper::FKT)&Wrapper::methodInfo,"","string","methodname","{}",0,1,MethodTable::STATIC,"Query method information");
	return dta_;
}

EnumTable* Wrapper::enumtable() const {
	return 0;
}

struct InitVoidWrapper {
	InitVoidWrapper() {
		WrapperManager::registerPrototype(new Wrapper(0));
	}
};
static InitVoidWrapper instanceOfVoidWrapper ;


IVRS_NAMESPACE_END
