#include <ivrs/wrapperclassinfo.h>

IVRS_NAMESPACE_BEGIN


#ifdef __GNUC__ 
#include <stdlib.h>
#include <string.h>


static int readnumber(const char*& cur) {
    char* endptr = 0;
    long len = strtol( cur, &endptr, 10 );
    cur = endptr;
    return (int)len;
}

const char* cleanRTTIName(const type_info* ti) {
	VRS_Error(ti,"no master class typeinfo");
	const char* cur = ti->name();
    char* clean = new char[256];
    char* out = (char*)clean;
    int tmpl = 0;
    bool ispointer = false;
    
    while (*cur) {
        if (*cur == 'N') {
            cur ++;
            if (*cur == 'S') { 
                cur += 2;
            } else {
                int len = readnumber(cur);
                cur += len;
            }
        } else if ( *cur == 'P' ) {
            ispointer = true;
            cur++;
	} else if (*cur == 'S') { 
	    cur++;
        } else if ( *cur == 'I' ) {
            *out = '<'; out++;
            tmpl++;
            cur++;
        } else if ( *cur == 'E' ) {
	    if (ispointer) {
                ispointer = false;
                *out = '*'; out++;
	    }
            if (tmpl>0) {
                *out = '>'; out++;
                tmpl--;
            }
            cur++;
        } else if ( *cur >= '0' && *cur <= '9') {
            int len = readnumber(cur);
            strncpy(out,cur,len);
            out+= len; 
            cur+= len;
        } else if (*cur == 'v') {
            strncpy(out,"void",4);
            out+=4;
            cur++;
        } else if ( *cur == 'j' ) {
            strncpy(out,"unsignedint",11);
            out+=11;
            cur++;
        } else if ( *cur == 'i' ) {
            strncpy(out,"int",3);
            out+=3;
            cur++;
        } else if ( *cur == 'f' ) {
            strncpy(out,"float",5);
            out+=5;
            cur++;
        } else if ( *cur == 's' ) {
            strncpy(out,"string",6);
            out+=6;
            cur++;
        } else if ( *cur == 'd' ) {
            strncpy(out,"double",6);
            out+=6;
            cur++;
        } else {
            return "ERROR";
        }
    }
    if (ispointer) { *out = '*'; out++; }
    *out = 0;
    
    if (!strcmp(clean, "Vector3Base<double>")) {
        strcpy(clean, "Vector");
    }

    if (!strcmp(clean, "Matrix4x4Base<double>")) {
        strcpy(clean, "Matrix");
    }

    return clean;
}


#else // windows

//
// cleanRTTIName
//
const char* cleanRTTIName(const type_info* ti) {
    VRS_Error(ti,"no master class typeinfo");
	const char* cur = ti->name();
    
	char* clean = new char[256];    
    static char* tokens[] = {"class ", "LDX::", "VRS::"};
	static const char* mystr = "class std::basic_string<char,struct std::char_traits<char>,class std::allocator<char> >";
    static int sl = strlen (mystr);

	int tokenlen = sizeof (tokens)/sizeof(char*);
    char* dst = clean;        
    while (*cur) {
		
		if (!strncmp(cur,mystr,sl)) { 
			strcpy(dst,"string");
			dst+= 6; cur += sl; 
		}

        for (int i = 0; i < tokenlen; i++) {
            int tl = strlen(tokens[i]);
            if (!strncmp(cur,tokens[i],tl)) { cur += tl; }
        }
        if ((*cur) == ' ') { cur++; }        
        
        (*dst) = (*cur);
        dst++; cur++;
    }
    (*dst) = 0;    
    
    std::string cleanStr = std::string(clean);
    std::string::size_type pos = std::string::npos;
    while ((pos = cleanStr.rfind("Vector3Base<double>", pos-1)) != std::string::npos) {
        cleanStr.replace(pos, 19, "Vector");
    }
    pos = std::string::npos;
    while ((pos = cleanStr.rfind("Matrix4x4Base<double", pos-1)) != std::string::npos) {
        cleanStr.replace(pos, 21, "Matrix");
    }
    const char* tmpres = cleanStr.c_str();
    strcpy(clean, tmpres);

    return clean;
}
#endif

WrapperClassInfo::WrapperClassInfo(const type_info& master, 
                                   WrapperClassInfo* masterbaseinfo) 
                                   : master_(&master), masterbaseinfo_(masterbaseinfo), name_(cleanRTTIName(master_))  {
    VRS_Error(master_,"no master class typeinfo");
    VRS_Error(name_,"no master class name");
}


IVRS_NAMESPACE_END
