#include <ivrs/wrappermanager.h>

IVRS_NAMESPACE_BEGIN


////////////////////////////////////
// Prototype management
////////////////////////////////////

Str2WrpMap* str2prt() {
	static Str2WrpMap s;
	return &s;
}
Type2WrpMap* type2prt() {
	static Type2WrpMap s;
	return &s;
}
void WrapperManager::registerPrototype(Wrapper* obj) {
    VRS_Error(obj,"no wrapper prototype");
    
    WrapperClassInfo* ci = obj->wrapperInfo();
    VRS_Error(ci,"no wrapper class info for " << obj);

	str2prt()->insert(Str2WrpMap::value_type(ci->masterClassName(),obj));	
    type2prt()->insert(Type2WrpMap::value_type(ci->masterClassTypeInfo(),obj));	
}

Wrapper* WrapperManager::wrapperPrototype(const char* classname) {
    VRS_Error(classname,"no classname");
	Str2WrpMap::iterator i = str2prt()->find(classname);	
	return (i != str2prt()->end()) ? i->second : 0;

}

WrpTableItr WrapperManager::allPrototypes() {
	return WrpTableItr(str2prt()->begin(),str2prt()->end());	
}

Wrapper* WrapperManager::wrapperPrototype(const type_info* type) {
    VRS_Error(type,"no prototype type_info");
	Type2WrpMap::iterator i = type2prt()->find(type);	
	return (i != type2prt()->end()) ? i->second : 0;
}

////////////////////////////////////
// wrapper management
////////////////////////////////////


static Str2WrpMap* str2wrp() {
	static Str2WrpMap s;	
	return &s;
}

static Void2WrpMap* void2wrp() {
	static Void2WrpMap s;
	return &s;
}

WrpTableItr WrapperManager::allObjects() {
	return WrpTableItr(str2wrp()->begin(),str2wrp()->end());		
}

void WrapperManager::add(Wrapper* obj) {
    VRS_Error(obj,"no wrapper to add");	
    str2wrp()->insert(Str2WrpMap::value_type(obj->name(),obj));

    VRS_Error(obj->obj(),"no wrapped obj");	
    void2wrp()->insert(Void2WrpMap::value_type(obj->obj(),obj));	
}	

bool WrapperManager::check(Wrapper* prototype, const type_info* desiredtype) {	
    VRS_Error(prototype,"no wrapper");
    
    // desiredtype == 0 ==> doesn't matter what type
    if (!desiredtype) return true;

    // check if wrapper is of desired type    
	WrapperClassInfo* ci = prototype->wrapperInfo();
	VRS_Error(ci,"wrapper has no WrapperClassInfo");

	while (ci) {
		// compare RTTI typeid
		if ((*(ci->masterClassTypeInfo())) == (*desiredtype) ) { return true; }
		ci = ci->masterBaseInfo();
	}	

    //VRS_Warning(0,"could not find desired type " << desiredtype->name());
	return false;
}

Wrapper* WrapperManager::find(const char* wrappername, const type_info* desiredtype) {
    VRS_Error(wrappername,"no wrappername to find");

	Str2WrpMap::iterator i = str2wrp()->find(wrappername);
    if (i == str2wrp()->end()) { return 0; }    

	Wrapper* wrp = i->second;
    VRS_Error(wrp,"wrappername has no wrapper");    
        
    return (check(wrp,desiredtype) ? wrp : 0);
}

Wrapper* WrapperManager::find(const void* obj, const type_info* desiredtype) {
    VRS_Error(obj,"no object to find");

	Void2WrpMap::iterator i = void2wrp()->find(obj);
	if (i == void2wrp()->end() ) { return 0; } 
    
    Wrapper* wrp = i->second;
    VRS_Error(wrp,"wrappername has no wrapper");
    
    return (check(wrp,desiredtype) ? wrp : 0);
}

void WrapperManager::remove(Wrapper* wrp) {
    VRS_Error(wrp,"no wrapper");
	str2wrp()->erase(wrp->name());		
	void2wrp()->erase(wrp->obj());
    wrp->destroyObject();
	delete wrp;
}
	
void WrapperManager::clear() {	
	for (SWI i = str2wrp()->begin();i != str2wrp()->end();i++) {				
		delete i->second;
	}
	str2wrp()->clear();
	void2wrp()->clear();    
}


IVRS_NAMESPACE_END