# load VRS package
source initvrs.tcl


#######################################################################
#
# IDE for 'iVRS'-Applications
#
#
#######################################################################


namespace eval devADS {


toplevel .env
wm title .env " iVRS "
wm geometry .env 230x55+5+5
wm resizable .env 1 1
wm withdraw .
wm withdraw .env


# include package for BWidgets
namespace inscope :: package require BWidget


################################### define global variables ################################

# variable for filename
variable fileName "new 1"


# helper variable for filename
variable fileName_old "new 1"


# list contains all open filenames 
variable openFiles ""
variable openFiles [linsert $openFiles 0 "<$fileName>"]


# global counter for 'no name' documents
variable counter 1


# variable for state of "GO"-Button
variable goStatus 0


# variable for the contents (application state) of status bar
variable appState ""


# define global list of classnames in VRS 
variable VRSClasses [VRS info classes]


# define global list of TCL statements (for syntax highlighting)
regsub -all {\s} [lsort -unique $VRSClasses] {|} tmpVRS
regsub -all "$tmpVRS|\\.\\S*" [info commands] {} TCLCommands
regsub -all {\s+} $TCLCommands { } TCLCommands
unset tmpVRS
set TCLCommands [lsort -unique $TCLCommands]





# define tag array (for syntax highlighting)
global tagArray tagSymbolArray

# associate tags to array elements
foreach a $TCLCommands { set tagArray($a) Tcl }
foreach a $VRSClasses { set tagArray($a) Vrs }

# associate some special iVRS commands explicitly
set tagArray(new) Vrs 
set tagArray(delete) Vrs 
set tagArray(VRS) Vrs

# associate tags zu symbols
set tagSymbolArray($) Vari
set tagSymbolArray(#) Com
set tagSymbolArray(.) Nav
set tagSymbolArray(-) Tk





# variable contains path to 'Window'-Menu
variable winMenu .env.menubar.win.m


# variable contains path to 'last recently opened'-Menu
variable lruMenu .env.menubar.file.m.lrused


# helper variable for 'Ctrl-Tab' mechanism in IDE
variable wort_old ""


# global variable for cursor position
variable cursor "line 1"


# if file exists read user's configuration from there, else create file with default values
if {[file exists "config.txt"]} {

	source "config.txt"

} else {

	# variable for color of syntax highlighting of VRS classes
	variable synVrs "sienna4"

	# variable for color of syntax highlighting of Tcl commands
	variable synTcl "blue"

	# variable for color of syntax highlighting of comments
	variable synCom "green4"

	# variable for color of syntax highlighting of VRS enums
	variable synEnu "DeepPink3"

	# variable for color of syntax highlighting of Tk GUI element paths
	variable synNav "LavenderBlush4"

	# variable for color of syntax highlighting of Tk options
	variable synTk "yellow4"

	# variable for color of syntax highlighting of variable names
	variable synVari "SteelBlue2"

	# variable for color of search results
	variable searchCol "red1"

	# variable for color of info highlighting
	variable infoCol "red4"

	# variable for color of error highlighting
	variable errorCol "red2"

	# variable for font size
	variable size "9"

	# variable (bool) for enabling/disabling of syntax highlighting of VRS classes
	variable checkVrs "1"

	# variable (bool) for enabling/disabling of syntax highlighting of Tcl commands
	variable checkTcl "1"

	# variable (bool) for enabling/disabling of syntax highlighting of comments
	variable checkCom "1"

	# variable (bool) for enabling/disabling of syntax highlighting of GUI hierarchy paths
	variable checkNav "1"

	# variable (bool) for enabling/disabling of syntax highlighting of VRS enums
	variable checkEnu "1"

	# variable (bool) for enabling/disabling of syntax highlighting of Tk options
	variable checkTk "1"

	# variable (bool) for enabling/disabling of syntax highlighting of variables
	variable checkVari "1"

	# variables contain filenames of recent files
	variable lruOne ""
	variable lruTwo ""
	variable lruThree ""
	variable lruFour ""	

	# list contains all filenames of recent files
	variable fileList ""

	# variable (bool) for enabling/disabling of syntax highlighting
	variable high 1
}


# clean list of recent files
variable continue 1
while {$continue == "1"} {

	variable test [lsearch $fileList ""]

	if {$test != -1} {
							
		# remove old entry
		set fileList [lreplace $fileList $test $test]
	
	} else {

		set continue 0
	}
} 



################################# define GUI components ###########################


wm title .env " iVRS -- $fileName"


# Menu 

frame .env.menubar -border 1 -relief raised
menubutton .env.menubar.file -text "File" -menu .env.menubar.file.m -underline 0
menubutton .env.menubar.help -text "Help" -menu .env.menubar.help.m -underline 0
menubutton .env.menubar.opt -text "Options" -menu .env.menubar.opt.m -underline 0
menubutton .env.menubar.eval -text "Evaluate" -menu .env.menubar.eval.m -underline 0
menubutton .env.menubar.set -text "Settings" -menu .env.menubar.set.m -underline 0
menubutton .env.menubar.win -text "Window" -menu $winMenu -underline 0


menu .env.menubar.file.m -tearoff false
	.env.menubar.file.m add command -label "New file         Ctrl+N" -command "devADS::NewFile"
	.env.menubar.file.m add separator	
	.env.menubar.file.m add command -label "Open file        Ctrl+O" -command "devADS::OpenFile"
	.env.menubar.file.m add separator	
	.env.menubar.file.m add command -label "Close file" -command "devADS::CloseFile"
	.env.menubar.file.m add separator	
	.env.menubar.file.m add command -label "Save file        Ctrl+S" -command "devADS::SaveFile"
	.env.menubar.file.m add command -label "Save file as" -command "devADS::SaveFileAs"
	.env.menubar.file.m add separator
	.env.menubar.file.m add cascade -label "Recent files " -menu $lruMenu
	.env.menubar.file.m add separator	
	.env.menubar.file.m add command -label "Exit.." -command "devADS::ExitApp"

menu .env.menubar.file.m.lrused -tearoff false
	.env.menubar.file.m.lrused add command -label "1   $lruOne" -command "devADS::OpenLRU {$lruOne}"
	.env.menubar.file.m.lrused add command -label "2   $lruTwo" -command "devADS::OpenLRU {$lruTwo}"
	.env.menubar.file.m.lrused add command -label "3   $lruThree" -command "devADS::OpenLRU {$lruThree}"
	.env.menubar.file.m.lrused add command -label "4   $lruFour" -command "devADS::OpenLRU {$lruFour}"

menu .env.menubar.eval.m -tearoff false
	.env.menubar.eval.m add check -label "Evaluate code" -variable devADS::goStatus -command "devADS::EvalCode"
	.env.menubar.eval.m add command -label "Stop code evaluation" -command "devADS::StopEvalCode"
	.env.menubar.eval.m add separator
	.env.menubar.eval.m add command -label "Show console" -command { devADS::ShowConsole }

menu .env.menubar.opt.m -tearoff false
	.env.menubar.opt.m add check -label "Syntax highlighting on/off" -variable devADS::high -command "devADS::SetHighlight 1.0 end"
	.env.menubar.opt.m add separator
	.env.menubar.opt.m add command -label "Show all VRS classes" -command "devADS::ShowVRS"
	.env.menubar.opt.m add command -label "Show all Tcl/Tk commands" -command "devADS::ShowTCL"
	.env.menubar.opt.m add separator
	.env.menubar.opt.m add command -label "Search                    Ctrl+F" -command "devADS::Search"

menu .env.menubar.set.m -tearoff false
	.env.menubar.set.m add cascade -label "Change color of ... " -menu .env.menubar.set.m.col
	.env.menubar.set.m add cascade -label "Change size of text " -menu .env.menubar.set.m.siz
	.env.menubar.set.m add cascade -label "Syntax highlighting " -menu .env.menubar.set.m.syn
	
menu .env.menubar.set.m.col -tearoff false
	.env.menubar.set.m.col add cascade -label "Syntax highlighting of ... " -menu .env.menubar.set.m.col.syntax
	.env.menubar.set.m.col add command -label "Search highlighting" -command "devADS::ChangeColor devADS::searchCol devADS::codeWin search 0"
	.env.menubar.set.m.col add command -label "Info highlighting" -command "devADS::ChangeColor devADS::infoCol devADS::infoWin info 1"
	.env.menubar.set.m.col add command -label "Error highlighting" -command "devADS::ChangeColor devADS::errorCol devADS::codeWin error 0"

menu .env.menubar.set.m.col.syntax -tearoff false
	.env.menubar.set.m.col.syntax add command -label "VRS classes" -command "devADS::ChangeColor devADS::synVrs devADS::codeWin Vrs 1"
	.env.menubar.set.m.col.syntax add command -label "Tcl/Tk commands" -command "devADS::ChangeColor devADS::synTcl devADS::codeWin Tcl 1"
	.env.menubar.set.m.col.syntax add command -label "Comments" -command "devADS::ChangeColor devADS::synCom devADS::codeWin Com 1"
	.env.menubar.set.m.col.syntax add command -label "GUI hierarchy paths" -command "devADS::ChangeColor devADS::synNav devADS::codeWin Nav 1"
	.env.menubar.set.m.col.syntax add command -label "VRS enums" -command "devADS::ChangeColor devADS::synEnu devADS::codeWin Enu 1"
	.env.menubar.set.m.col.syntax add command -label "Tk options" -command "devADS::ChangeColor devADS::synTk devADS::codeWin Tk 1"
	.env.menubar.set.m.col.syntax add command -label "Variables" -command "devADS::ChangeColor devADS::synVari devADS::codeWin Vari 1"

menu .env.menubar.set.m.siz -tearoff false
	
	# create menu entries
	foreach number { 6 7 8 9 10 11 12 13 14 } {
		
		.env.menubar.set.m.siz add radio -label "size $number" -variable devADS::size -value $number -command "devADS::ChangeSize"
	}
	
	
menu .env.menubar.set.m.syn -tearoff false
	.env.menubar.set.m.syn add check -label "VRS classes" -variable devADS::checkVrs -command "devADS::ChangeHighlight Vrs"
	.env.menubar.set.m.syn add check -label "Tcl/Tk commands" -variable devADS::checkTcl -command "devADS::ChangeHighlight Tcl"
	.env.menubar.set.m.syn add check -label "Comments" -variable devADS::checkCom -command "devADS::ChangeHighlight Com"
	.env.menubar.set.m.syn add check -label "GUI hierarchy paths" -variable devADS::checkNav -command "devADS::ChangeHighlight Nav"
	.env.menubar.set.m.syn add check -label "VRS enums" -variable devADS::checkEnu -command "devADS::ChangeHighlight Enu"
	.env.menubar.set.m.syn add check -label "Tk options" -variable devADS::checkTk -command "devADS::ChangeHighlight Tk"
	.env.menubar.set.m.syn add check -label "Variables" -variable devADS::checkVari -command "devADS::ChangeHighlight Vari"

menu $winMenu -tearoff false
	$winMenu add radio -label "$fileName" -variable devADS::fileName -value $fileName -command "devADS::ChangeWindow"
	
menu .env.menubar.help.m -tearoff false
	.env.menubar.help.m add command -label "iVRS help" -command "devADS::ViewiVRSHelp"
	.env.menubar.help.m add command -label "IDE help" -command "devADS::ViewIDEHelp"	
	.env.menubar.help.m add separator 
	.env.menubar.help.m add command -label "About..." -command "devADS::About"

pack .env.menubar.file .env.menubar.eval .env.menubar.opt .env.menubar.set .env.menubar.win -side left
pack .env.menubar.help -side right
pack .env.menubar -fill x -side top




# define icons for button elements in toolbar
image create photo greenpfeil -format gif -file "pics/pfeil_green.gif"
image create photo pfeil -format gif -file "pics/pfeil.gif"
image create photo stop -format gif -file "pics/stop.gif"
image create photo consol -format gif -file "pics/console.gif"

# define icons for tree elements
image create photo kugel -format gif -file "pics/uml.gif"
image create photo template -format gif -file "pics/umlTempl.gif"



# create toolbar
frame .env.tb 
pack .env.tb -side top -anchor w -padx 5 -pady 3

# create toolbar 1
frame .env.tb.tool1
pack .env.tb.tool1 -side left -anchor w

button .env.tb.tool1.new -image [Bitmap::get new] -command { devADS::NewFile }
button .env.tb.tool1.open -image [Bitmap::get open] -command { devADS::OpenFile }
button .env.tb.tool1.save -image [Bitmap::get save] -command { devADS::SaveFile }
pack .env.tb.tool1.new .env.tb.tool1.open .env.tb.tool1.save -side left -anchor w -padx 3 -pady 3

# create toolbar 2
frame .env.tb.tool2
pack .env.tb.tool2 -side left -anchor w -padx 10


checkbutton .env.tb.tool2.go -indicatoron false -variable devADS::goStatus -image pfeil -command { devADS::EvalCode }
button .env.tb.tool2.stop -image stop -command { devADS::StopEvalCode }
button .env.tb.tool2.console -image consol -state disabled -command { devADS::ShowConsole }
pack .env.tb.tool2.stop .env.tb.tool2.go .env.tb.tool2.console -side left -anchor w -padx 3 -pady 3

# set state of menu entry for console disabled
.env.menubar.eval.m entryconfigure "Show console" -state disabled



# define flexible Widget
PanedWindow .env.pane -side top

# create flexible frames
variable li [.env.pane add -weight 0] 
variable re [.env.pane add -weight 1] 

pack .env.pane -fill both -expand yes




# create tree in left flexible frame

# create scrollable area for tree 
ScrolledWindow $li.canvas -relief groove

# create tree
Tree $li.canvas.tree -padx 25 -deltay 15 -relief groove

# put tree into scrollable area
$li.canvas setwidget $li.canvas.tree
pack $li.canvas -side top -fill both -expand true -padx 5 -pady 5


# bind double click on any tree element to info-display-procedure
$li.canvas.tree bindText <Double-Button-1> devADS::ShowIvrsClassInfo



variable liste ""

foreach class $VRSClasses {

	# get parent class
	variable parent [VRS info parentclass $class]
		
	# get classes, which do not have any parent class
	if {$parent == "void*"} {

		# create entry with parent 'root'
		$li.canvas.tree insert end root $class -text "$class" -drawcross auto -image kugel
		
		# create list of those classes (without parent class)
		lappend liste $class
	}
}



############################ procedure to initialize class tree #####################

proc MakeTree {liste} {

	variable li
	
	# for each node of list get subtrees
	foreach class $liste {

		variable newList ""

		# create list of childclasses
		variable children [VRS info childclasses $class]

		if {$children != ""} {

			# sort list
			set children [lsort $children]

			# create entries
			foreach kind $children {								

				# check if class is kind of template
				if {[regexp {<} $kind]} {
															
					regexp {([^<]+)} $kind dummie general

					if {![$devADS::li.canvas.tree exists "$general<T>"]} {

						# create generic template entry
						$li.canvas.tree insert end $class "$general<T>" -text "$general<T>" -drawcross auto -image template
					} 

					# insert class with generic template entry as parent
					$li.canvas.tree insert end "$general<T>" $kind -text "$kind" -drawcross auto -image kugel

				} else {

					# insert normally
					$li.canvas.tree insert end $class $kind -text "$kind" -drawcross auto -image kugel
					lappend newList $kind
				}
			}
			
			# sort list
			set newList [lsort $newList]
			
			# recursive step - get subtrees of those nodes
			MakeTree $newList		
		}
	}
}


##################################### end procedure ###############


# sort list
set liste [lsort $liste]

# call init procedure
MakeTree $liste


# create search entry
frame $li.frm 
pack $li.frm -side bottom -anchor s -fill x

label $li.frm.lab -text "search for classname in tree:"
entry $li.frm.entry

pack $li.frm.lab -side top -anchor w -expand true -padx 5 -pady 5
pack $li.frm.entry -side top -anchor w -expand true -padx 5 -pady 5 -fill x





# create flexible widget
PanedWindow $re.pane -side left

# create flexible frames
variable ob [$re.pane add] 
variable un [$re.pane add] 

pack $re.pane -fill both -expand yes



# create flexible widget
PanedWindow $un.pane -side top

# create flexible frames
variable unli [$un.pane add -weight 3] 
variable unre [$un.pane add -weight 2] 

pack $un.pane -fill both -expand yes



#############	define variables containing the paths to the 3 main windows  #############


# info window
variable infoWin $unre.textFrame.info


# output window
variable outWin $unli.textFrame.output


# code window
variable codeWin $ob.quelltext


################################### end define ###########################################



# upper window (code window)

text $codeWin -setgrid true -wrap none -width 113 -height 25 -font "courier $size"\
		-xscrollcommand [list $ob.scrollX set]\
		-yscrollcommand [list $ob.scrollyBottom set] -relief groove
scrollbar $ob.scrollX -orient horizontal -command [list $codeWin xview]
scrollbar $ob.scrollyBottom -orient vert -command [list $codeWin yview]

grid $codeWin $ob.scrollyBottom -sticky news
grid $ob.scrollX -sticky news
grid rowconfigure $ob 0 -weight 1
grid columnconfigure $ob 0 -weight 1




# info window

frame $unre.textFrame -borderwidth 5
pack $unre.textFrame -side bottom -fill both -expand true
text $infoWin -setgrid true -wrap word -width 50 -height 5 -font "courier $size"\
		-yscrollcommand "$unre.textFrame.scrollyRight set" -relief groove
scrollbar $unre.textFrame.scrollyRight -orient vert -command "$infoWin yview"
pack $unre.textFrame.scrollyRight -side right -fill y
pack $infoWin -side left -fill both -expand true

label $unre.lab -text "information: "
pack $unre.lab -side left




# output window

frame $unli.textFrame -borderwidth 5
pack $unli.textFrame -side bottom -fill both -expand true
text $outWin -setgrid true -wrap word -width 50 -height 5 -font "courier $size"\
		-yscrollcommand "$unli.textFrame.scrollyLeft set" -relief groove
scrollbar $unli.textFrame.scrollyLeft -orient vert -command "$outWin yview"
pack $unli.textFrame.scrollyLeft -side right -fill y
pack $outWin -side left -fill both -expand true

label $unli.lab -text "output: "
pack $unli.lab -side left



# statusbar (displays cursor position and application state)

frame .env.statusbar -border 1 -relief raised
pack .env.statusbar -side bottom -fill x -anchor s

label .env.statusbar.lab2 -textvariable devADS::cursor -width 10
pack .env.statusbar.lab2 -side right -padx 15

label .env.statusbar.lab1 -textvariable devADS::appState
pack .env.statusbar.lab1 -side left -padx 15


################################ define tags ################################################################


$codeWin tag configure wholeText -lmargin2 10i
$codeWin tag add wholeText 1.0 end

# tags for syntax highlighting
foreach var { Tcl Vrs Nav Com Enu Tk Vari } {

	# associate variable varUp with syn... (because of double dollar sign)
	upvar devADS::syn$var varUp
	
	# define tag
	$codeWin tag configure $var -foreground $varUp
}
 
# tags for backgroundcolors (error and search)
$codeWin tag configure search -background $searchCol
$codeWin tag configure error -background $errorCol

# tag for color of headings in info window
$infoWin tag configure info -foreground $infoCol

# tag for color in output window
$outWin tag configure errorOut -foreground $errorCol


############################################## bindings ########################################################


# bind info mechanism to code window
bind $codeWin <Button-3> "event generate $codeWin <Control-Key-Tab>"

bind $codeWin <Control-Key-Tab> {
	
	$devADS::codeWin mark set pos insert
	variable continue 1
	variable flag_one 0
	variable flag_two 0
	
	# while going backwards, only go until line starts
	while {[$devADS::codeWin compare pos != "pos linestart"] && $continue} {

		variable zeichen [$devADS::codeWin get pos]
		
		if {[regexp {\s} $zeichen]} {
			
			$devADS::codeWin mark set pos "pos -1 chars"
			
		} else {
			
			set continue 0
		}
	}		

	variable wort [$devADS::codeWin get "pos wordstart" "pos wordend"]
	
	variable endergebnis ""
	variable ergebnis ""
	
	foreach test $devADS::VRSClasses {
			
		regexp "$wort.*" $test ergebnis
		
		if {$ergebnis != ""} {

			lappend endergebnis $ergebnis
			set ergebnis ""
			
		}  
	}

	
	# multiple choice
	if {[llength $endergebnis] > 1} {
		
		# because it should be possible to show 'Texture' properties - although there is a class named 'TextureGL': 
		# repeated choices will be treated differently
		if {$wort != $devADS::wort_old} {

			set devADS::wort_old $wort
			set flag_one 0

		} else {

			set flag_one 1
		}
		
		# if choice is repeated (flag is set)	
		if {$flag_one == 1} {

			foreach thing $endergebnis {
				
				if {$wort == $thing} {
					
					set flag_two 1
				}
			}
		
			# check, if '$wort' is a valid classname (e.g., 'Texture')	
			if {$flag_two == 1} {

				# display class information
				devADS::OutputClassInfo $wort
					
				break
			}
		}
		
			
		# else output of available classnames in info window
		$devADS::infoWin delete 1.0 end
		$devADS::infoWin tag remove info 1.0 end

		$devADS::infoWin insert 1.0 "available classes:"
		$devADS::infoWin tag add info 1.0 1.end
		$devADS::infoWin insert end "\n\n"
		
		foreach class $endergebnis {

			$devADS::infoWin insert end "$class\n"
		}
		
		# Popup-Menu
		if {[winfo exists .popup]} { destroy .popup }
		menu .popup -tearoff false
		foreach {class} $endergebnis {
			.popup add command -label $class -command "devADS::Insert $class ; destroy .popup"
		}
		
		tk_popup .popup [winfo pointerx .] [winfo pointery .]
		
		break
	}

	# classname non-ambiguous identified
	if {[llength $endergebnis] == 1} {
		
		# classname completely typed?
		if {$wort == $endergebnis} {
		
			# display class information
			devADS::OutputClassInfo $wort
			break
		} 
		
		# if not completely typed - complete automatically (UNIX-like)
		$devADS::codeWin delete "pos wordstart" "pos wordend"
		$devADS::codeWin insert "pos wordstart" $endergebnis
		break
	}
}			

			 
# bind doubleclick on word in info window to insert word in code window
bind $infoWin <Double-Button-1> {

	# get word clicked on
	variable auswahl [$devADS::infoWin get "insert wordstart" "insert wordend"]
	
	# call procedure
	devADS::Insert $auswahl
}	

# refresh global variable 'cursor'
bind $codeWin <Any-Key> {after idle devADS::RefreshCursor}


# check current line, if syntax highlighting is enabled
bind $codeWin <Key-Return> {

	# get only line info
	variable pos [lindex [split [$devADS::codeWin index insert] .] 0]
	
	if {$devADS::high == 1} {

		devADS::SetHighlight "$pos.0" "$pos.end"
	}

	after idle devADS::RefreshCursor
}

# check current line, if syntax highlighting is enabled
bind $codeWin <Key-space> {

	# get only line info
	variable pos [lindex [split [$devADS::codeWin index insert] .] 0]
	
	if {$devADS::high == 1} {

		devADS::SetHighlight "$pos.0" "$pos.end"
	}
}

# refresh global variable 'cursor'
bind $codeWin <ButtonRelease> {devADS::RefreshCursor}

# define shortcuts
bind .env <Control-Key-s> { .env.menubar.file.m invoke "Save file        Ctrl+S" }
bind .env <Control-Key-o> { .env.menubar.file.m invoke "Open file        Ctrl+O" }
bind .env <Control-Key-f> { .env.menubar.opt.m invoke "Search                    Ctrl+F" }
bind .env <Control-Key-n> { .env.menubar.file.m invoke "New file         Ctrl+N" }


# bind search entry (from tree) 
bind $li.frm.entry <Key-Return> {

	# get entry contents
	set wort [$devADS::li.frm.entry get]

	# check, if classname exists in tree
	if {[$devADS::li.canvas.tree exists $wort]} {

		devADS::MakeVisible $wort

		# make node visible and colored
		$devADS::li.canvas.tree see $wort
		$devADS::li.canvas.tree itemconfigure $wort -fill $devADS::searchCol
	}
}




################################	procedures	 #####################################


# procedure to evaluate code in code window (called from menu or GO button)
proc EvalCode {} {

	variable appState 

	# reconfigure GO-button
	.env.tb.tool2.go configure -image greenpfeil -state disabled 
	.env.menubar.eval.m entryconfigure "Evaluate code" -state disabled
	set appState "application running"

	# reset (enable) Console button and menu button
	.env.tb.tool2.console configure -state normal
	.env.menubar.eval.m entryconfigure "Show console" -state normal

	# call evaluation procedure
	devADS::Execute 
}


# procedure to stop evaluation (destroys slave interpreter)
proc StopEvalCode {} {

	variable goStatus
	variable appState

	# reconfigure GO-button
	.env.tb.tool2.go configure -image pfeil -state normal
	.env.menubar.eval.m entryconfigure "Evaluate code" -state normal
	set goStatus 0
	set appState "application stopped"

	# reset (disable) Console Button and menu button
	.env.tb.tool2.console configure -state disabled
	.env.menubar.eval.m entryconfigure "Show console" -state disabled

		
	# destroy slave interpreter
	if {[interp exists app]} { 
		
		# clean up VRS 
		app eval VRS cleanup
		interp delete app 
	}
}


# procedure to evaluate code 
proc Execute {} {
	
	variable codeWin 
	variable outWin 
	
	# eventually remove tags from code window and entries from output window
	$codeWin tag remove error 1.0 end
	$outWin tag remove errorOut 1.0 end
	$outWin delete 1.0 end

	# remind code (in file)
	variable inhalt [$codeWin get 1.0 end]

	# if file exists, it would be deleted
	file delete "temporary.txt" 
	
	variable out [open "temporary.txt" w]
	puts $out "$inhalt"
	close $out
		
	# destroy slave interpreter, if exists
	if {[interp exists app]} { 
		app eval VRS cleanup
		interp delete app 
	}
		
	# create interpreter (see procedure)
	CreateInterpreter app		

	# define aliases (for slave interpreter)
	interp alias app knopf {} .env.tool.console
	interp alias app menuknopf {} .env.menubar.eval.m

	# slave evaluates reminded code in file
	if {[catch {app eval [list source "temporary.txt"]; app eval [list wm protocol . WM_DELETE_WINDOW { knopf configure -state disabled; menuknopf entryconfigure "Show console" -state disabled; destroy . }] }]} {

		# get global error state
		global errorInfo
		
		# show error info
		$outWin insert 1.0 "$errorInfo\n\n"
		$outWin tag add errorOut 1.0 end
		$outWin see insert
		
		# move cursor to line, which contains error
		regexp {file "temporary.txt" line ([0-9]+)} $errorInfo match row
		$codeWin mark set insert "$row.0"
		$codeWin tag add error "$row.0" "$row.end"
		$codeWin see insert
		
		# call procedure to destroy interpreter and reset button states
		StopEvalCode
	} 

	# if file exists, it would be deleted
	file delete "temporary.txt" 
}



# procedure to create slave interpreter '$name'
proc CreateInterpreter {name} {

	# create
	interp create $name

	# init
	load {} Tk $name
	load {} VRS $name
}



# procedure to create a new document
proc NewFile {} {

	foreach var { fileName openFiles counter winMenu codeWin } {
		variable $var
	}
	
	# if state disabled, set state normal
	$codeWin configure -state normal

	# increment counter for unknown documents	
	set counter [expr $counter + 1]
	
	# refresh current filename variable 
	set fileName "new $counter"

	# insert into list
	set openFiles [linsert $openFiles 0 "<$fileName>"]
	
	# call procedure to change window (within code window)
	ChangeWindow

	# add entry to menu 'Window'
	$winMenu add radio -label "$fileName" -variable devADS::fileName -value $devADS::fileName -command "devADS::ChangeWindow"
}



# procedure to 'Save File As'
proc SaveFileAs {} {
	
	foreach var { fileName codeWin counter fileName_old openFiles winMenu } {
		variable $var
	}
	
	# is it an unknown file?
	variable test ""
	regexp {^new [[:digit:]]+$} $fileName test

	set fileName [tk_getSaveFile]

	if {$fileName != ""} {
		
		variable inhalt [$codeWin get 1.0 end]
		
		# remove last '\n'
		regsub {\n$} $inhalt {} inhalt

		if [catch {open $fileName w} out] {
			
			tk_messageBox -type ok -message "an error occurred while opening $fileName " -icon error

		} else {
		
			puts $out $inhalt
			close $out

			# find respective list element
			variable test [lsearch -regexp $openFiles "^<$fileName_old>.*"]
		
			if {$test != -1} {

				# and replace by one with new name
				set openFiles [lreplace $openFiles $test $test "<$fileName>$inhalt"]
			}

			# remove respective menu entry ...
			$winMenu delete "$fileName_old"
		
			# ... and add entry with new name
			$winMenu add radio -label "$fileName" -variable devADS::fileName -value $devADS::fileName -command "devADS::ChangeWindow"	

			# decrement counter for unknown files
			if {$test != ""} {

				set counter [expr $counter - 1]
			}

			# refresh helper variable with new file name	
			set fileName_old $fileName

			wm title .env " iVRS -- $fileName"
		}

	} else {
		
		# reset current file name (because there was no saving action)
		set fileName $fileName_old
	}
	
	focus .env
}


# procedure to 'Save File'
proc SaveFile {} {
	
	variable fileName 
	variable codeWin

	variable test ""

	# save unknown file? - then give to 'SaveAs'
	regexp {^new [[:digit:]]+$} $fileName test

	if {$test == ""} {
		
		variable inhalt [$codeWin get 1.0 end]
		
		if [catch {open $fileName w} out] {
			
			tk_messageBox -type ok -message "an error occurred while opening $fileName " -icon error

		} else {
		
			puts $out $inhalt
			close $out
		}

	} else {

		SaveFileAs
	}
}


# procedure to 'Open File'
proc OpenFile {} {
	
	variable fileName 
	variable codeWin 
	variable fileName_old
	
	# in case of state was disabled
	$codeWin configure -state normal
	
	set fileName [tk_getOpenFile]
	
	if {$fileName != ""} {
		
		# call open procedure
		Open_inGeneral "$fileName"
		
	} else {
		
		# reset current file name (because there was no saving action)
		set fileName $fileName_old
	}

	focus .env
}


# procedure to 'Close File'
proc CloseFile {} {

	foreach var { fileName codeWin outWin infoWin openFiles counter fileName_old winMenu high } {
		
		variable $var
	}

	# close unknown file? - then decrement counter
	variable test ""
	regexp {^new [[:digit:]]+$} $fileName test

	if {$test != ""} {

		set counter [expr $counter - 1]
	
	} else {
	
		# ask for saving

		toplevel .askForSave
		wm title .askForSave " save contents ..." 
		wm geometry .askForSave 300x110+140+220
		wm resizable .askForSave 0 0

		# GUI
			
		frame .askForSave.oben -borderwidth 15
		frame .askForSave.unten
		pack .askForSave.oben .askForSave.unten -side top

		label .askForSave.oben.info -text "Attention:	Do you want to save before closing the file?"
		frame .askForSave.unten.li -borderwidth 15
		frame .askForSave.unten.re -borderwidth 15
		pack .askForSave.unten.li .askForSave.unten.re -side left
		button .askForSave.unten.re.no -text "NO" -command "destroy .askForSave" -width 10 -height 1
		button .askForSave.unten.li.yes -text "YES" -command "devADS::SaveFile; destroy .askForSave" -width 10 -height 1
		pack .askForSave.oben.info -side top
		pack .askForSave.unten.li.yes -side left
		pack .askForSave.unten.re.no  -side right
			
		# fix and wait
		grab .askForSave
		tkwait window .askForSave		
	}	
	
	# empty all windows
	$codeWin delete 1.0 end
	$infoWin delete 1.0 end
	$outWin delete 1.0 end
	
	# remove current document from list
	variable test [lsearch -regexp $openFiles "^<$fileName>.*"]
		
	if {$test != -1} {

		set openFiles [lreplace $openFiles $test $test]
	}

	# remove menu entry
	$winMenu delete "$fileName"	
		
	if {[llength $openFiles] != 0} {
		
		# display first document in list
		variable next [lindex $openFiles 0]	
		
		# get contents
		variable inhalt ""
		regexp {^<(.+?)>(.*)$} $next dummie fileName inhalt

		# display contents
		$codeWin insert end $inhalt

		# in case of enabled syntax highlighting
		if {$high == 1} {
			
			SetHighlight 1.0 end
		}

		# adjust helper variable
		set fileName_old $fileName

		wm title .env " iVRS -- $fileName"
	
	} else {
		
		$codeWin configure -state disabled
		wm title .env " iVRS "
		
		# because there is no document open anymore
		set fileName ""
		set fileName_old ""
	}
}


# procedure to do syntax highlighting
proc SetHighlight {start end} {

	foreach var { high codeWin checkTcl checkVrs checkCom checkNav checkEnu checkTk checkVari } {
		
		# make visible 
		variable $var
	}	 		

	if {$high == 0} {
		
		foreach var { Com Tcl Vrs Nav Enu Tk Vari } {
		
			# put off syntax highlighting
			$codeWin tag remove $var $start $end
		}
			
	} else {
	
		global devADS::tagArray devADS::tagSymbolArray
		
		# set starting point
		set si $start
						
		while {1} {	
			
			# look for 'dividing characters' (e.g., Space) and interpret all between two of them as a word
			set res [$codeWin search -count length -regexp {([^\s\t\n\r\[\]\"\}\{]+)} $si $end]
			
			# end
			if {$res == ""} { break } 
			
			# get index of wordend
			set wordEnd [$codeWin index "$res + $length chars"]

			# get word
			set word [$codeWin get $res $wordEnd] 

			# get first character of word
			set firstOfWord [string index $word 0] 
					
			# check, if there is an array entry associated to the word
			if {[info exists devADS::tagArray($word)]} {	
				
				# associate variable 'test' to 'devADS::check$...' (because of $$)
				upvar devADS::check$devADS::tagArray($word) test

				if {$test == "1"} {
														
					# bind word to tag
					$codeWin tag add $devADS::tagArray($word) $si "$wordEnd"
				}

				# refresh starting index for syntax highlighting
				set si [$codeWin index $wordEnd]
							
			} elseif {[info exists devADS::tagSymbolArray($firstOfWord)]} {

				# is first character a special mentioned character?
				if {$firstOfWord == "#"} { 

					if {$devADS::checkCom == "1"} {
					
						# bind line to comment tag 
						$codeWin tag add $devADS::tagSymbolArray($firstOfWord) $si "$si lineend"
					}
					
					# refresh starting index
					set si [$codeWin index "$si lineend + 1 chars"]

				} elseif {$firstOfWord == "-"} {

					# if there's no digit after "-" 
					if {![string is digit [string index $word 1]]} {
						
						if {$devADS::checkTk == "1"} {
						
							# interpret as option
							$codeWin tag add $devADS::tagSymbolArray($firstOfWord) $si $wordEnd
						}
					}
					
					# refresh starting index
					set si [$codeWin index $wordEnd]

				} elseif {$firstOfWord == "."} {

					# if there's no digit after "." 
					if {![string is digit [string index $word 1]]} {
						
						if {$devADS::checkNav == "1"} {
						
							# interpret as hierarchy path
							$codeWin tag add $devADS::tagSymbolArray($firstOfWord) $si $wordEnd
						}
					}
					
					# refresh starting index
					set si [$codeWin index $wordEnd]

				} else {
					
					# associate variable 'test' to 'devADS::check$...' (because of $$)
					upvar devADS::check$devADS::tagSymbolArray($firstOfWord) test

					if {$test == "1"} {
					
						# bind to appropriate tag 
						$codeWin tag add $devADS::tagSymbolArray($firstOfWord) $si $wordEnd	
					}		

					# refresh starting index
					set si [$codeWin index $wordEnd]
					
				}
			} else {
				
				if {[regexp {\:\:} $word]} {
					
					if {$devADS::checkEnu == "1"} {
					
						# interpret as enum 
						$codeWin tag add Enu $si "$wordEnd"
					}
				}
				
				# refresh starting index
				set si [$codeWin index $wordEnd]			
			}
		}
	}
}


# procedure to 'Search'
proc Search {} {

	variable codeWin	

	if {[winfo exists .search]} {

		wm deiconify .search
	
	} else {

		toplevel .search
		wm title .search " Search" 
		wm geometry .search 600x255+5+450
		wm resizable .search 0 1
		
		# GUI

		frame .search.links -borderwidth 10 
		frame .search.rechts -borderwidth 10 -width 100 -height 100
		frame .search.links.oben -borderwidth 10
		frame .search.links.unten -borderwidth 10

		frame .search.links.oben.oben -borderwidth 15 
		frame .search.links.oben.unten -borderwidth 15 
		frame .search.links.unten.oben -borderwidth 15 
		frame .search.links.unten.unten -borderwidth 15 

		frame .search.rechts.oben -borderwidth 5
		frame .search.rechts.mitte
		frame .search.rechts.unten -borderwidth 30  

		pack .search.links -side left
		pack .search.rechts -side right
		pack .search.links.oben .search.rechts.oben .search.rechts.mitte -side top
		pack .search.links.unten .search.rechts.unten -side bottom
		pack .search.links.oben.oben .search.links.oben.unten .search.links.unten.oben .search.links.unten.unten -side top

		entry .search.links.oben.unten.entsuch -width 50
		label .search.links.oben.oben.labsuch -text "look for: "
		entry .search.links.unten.unten.entrepl -width 50
		label .search.links.unten.oben.labrepl -text "replace by: "

		pack .search.links.oben.oben.labsuch .search.links.unten.oben.labrepl -side left
		pack .search.links.unten.unten.entrepl .search.links.oben.unten.entsuch -side left
	
		button .search.rechts.unten.exit -text "close" -command "wm withdraw .search; $codeWin tag remove search 1.0 end" -width 20 -height 2
		button .search.rechts.mitte.replace  -text "replace" -width 20 -height 2 -command "devADS::ReplaceWord"
		button .search.rechts.oben.search -text "search" -width 20 -height 2 -command "devADS::SearchWord"
	
		pack .search.rechts.oben.search .search.rechts.mitte.replace -side top
		pack .search.rechts.unten.exit -side bottom


		# bind RETURN to entry
		bind .search.links.oben.unten.entsuch <Key-Return> {

			devADS::SearchWord
		}
	}
}


# procedure for 'search' command
proc SearchWord {} {
	
	variable codeWin
	
	$codeWin tag remove search 1.0 end

	# get entry contents
	variable wort [.search.links.oben.unten.entsuch get]
	
	# remind cursor position in code for start point
	variable helpA [$codeWin index insert]
	
	# remind cursor position in code for end point
	$codeWin mark set pos "insert -1 chars"
	variable helpE [$codeWin index pos]
		
	# start searching in code
	variable cnt 0
	variable indx [$codeWin search -count cnt -regexp -- "$wort" $helpA]
	
	if {$indx != ""} {

		# mark found word
		$codeWin tag add search $indx "$indx +$cnt chars"
		
		# move cursor behind word 
		$codeWin mark set insert "$indx +$cnt chars"

		# make found word visible 
		$codeWin see insert

	} else {
		
		# show information window (search failed)
		if {[winfo exists .searchFailed]} {

			wm deiconify .searchFailed
	
		} else {

			toplevel .searchFailed
			wm title .searchFailed " your search failed..." 
			wm geometry .searchFailed 300x150+40+420
			wm resizable .searchFailed 0 0
			
			# GUI
			frame .searchFailed.oben -borderwidth 15
			frame .searchFailed.unten -borderwidth 15
			pack .searchFailed.oben .searchFailed.unten -side top

			label .searchFailed.oben.info -text "Info:	The word you searched for \n	doesn't exist (anymore)."
			button .searchFailed.unten.ok -text "OK" -command "destroy .searchFailed" -width 20 -height 2
			pack .searchFailed.oben.info .searchFailed.unten.ok -side bottom

			# fix and wait
			grab .searchFailed
			tkwait window .searchFailed
		}
	}
}
	

# procedure for 'replace' command
proc ReplaceWord {} {

	variable codeWin

	# get contents from 'replace by' entry
	variable wort [.search.links.unten.unten.entrepl get]

	foreach {start end} [$codeWin tag ranges search] {

		$codeWin delete $start $end
		$codeWin insert $start $wort
	}

	SearchWord
}


# procedure to 'Show all VRS classes'
proc ShowVRS {} {

	variable infoWin
	
	$infoWin delete 1.0 end
	$infoWin tag remove info 1.0 end
	
	# make global list visible
	variable VRSClasses

	$infoWin insert end "All classes available in iVRS: "
	$infoWin tag add info "insert linestart" "insert lineend"
	$infoWin insert end "\n\n"
	
	foreach class $VRSClasses {

		$infoWin insert end "$class\n"
	}
}


# procedure to 'Show all TCL commands'
proc ShowTCL {} {

	variable infoWin

	$infoWin delete 1.0 end
	$infoWin tag remove info 1.0 end
	
	# make global list visible
	variable TCLCommands

	$infoWin insert end "All commands available in iVRS-Tcl/Tk: "
	$infoWin tag add info "insert linestart" "insert lineend"
	$infoWin insert end "\n\n"
	
	foreach com $TCLCommands {

		$infoWin insert end "$com\n"
	}
}


# procedure for inserting the passed over word at the cursor position in source code window
# (if cursor is located after a word, replacing this);
# called by info windows binsing and pop-up menu commands
proc Insert {wort} {

	variable codeWin

	# set marking 'pos' to cursor position
	$codeWin mark set pos insert
		
	if {[$codeWin compare pos != "pos linestart"]} {

		variable zeichen [$codeWin get pos]
		
		# check if cursor is located after a word
		if {[regexp {\s} $zeichen]} {
			
			$codeWin mark set pos "pos -1 chars"
			set zeichen [$codeWin get pos]
			
			# reset 'pos', if cursor is not located after a word
			if {[regexp {\s} $zeichen]} {
			
				$codeWin mark set pos "pos +1 chars"
			} 
		} 
	}		
	
	# insert words into source code
	$codeWin delete "pos wordstart" "pos wordend"
	$codeWin insert insert $wort
}


# procedure for changing a color
proc ChangeColor { color win tag flag } {

	# 'color' contains the name of a variable containing the color
	upvar $color locCol

	# 'win' contains a name of a variable containing the control path
	upvar $win locWin

	variable var ""
	# call Tk color chooser dialog
	set var [tk_chooseColor -initialcolor $locCol]

	if {$var != ""} {
		
		# 'flag' decides whether background or foreground color should be changed
		if {$flag == 1} {

			# remember new color and reconfigure
			set $color $var
			$locWin tag configure $tag -foreground $var

		} else {

			# remember new color and reconfigure tag
			set $color $var
			$locWin tag configure $tag -foreground $var
		}
	}
}


# procedure for display cursor line refresh
proc RefreshCursor {} {

	variable cursor 
	variable codeWin
	
	# only this line is relevant
	variable pos [lindex [split [$codeWin index insert] .] 0]
	
	# update textvariable
	set cursor "line $pos"
}


# procedure contains cleanup activities, saving user's profile and end of application itsself 
proc ExitApp {} {
	
	# destroy interpreter, if appropriate
	if {[interp exists app]} { 
		app eval VRS cleanup 
		interp delete app
	}

	# clean up VRS system
	VRS cleanup 

	# if file exists, it would be deleted
	file delete "config.txt" 
	
	# create configuration file
	if [catch {open "config.txt" w} out] {
			
		tk_messageBox -type ok -message "an error occurred while creating configuration file " -icon error
	
	} else {
		
		foreach var { synVrs synTcl synCom synNav synEnu synTk synVari searchCol infoCol errorCol \
					size checkVrs checkTcl checkCom checkNav checkEnu checkTk checkVari high } {
		
			# make variable known
			variable $var

			# var contains name for real variable; associate varUp with this one
			upvar devADS::$var varUp
			
			# write to config file
			puts $out "variable $var $varUp\n"			
		} 
		
		foreach var { lruOne lruTwo lruThree lruFour } {
		
			# make variable known
			variable $var

			# var contains name for real variable; associate varUp with this one
			upvar devADS::$var varUp
			
			# write to config file
			puts $out "variable $var {$varUp}\n"				
		}		
		
		# write list of recently opened files to configuration file
		puts $out "variable fileList \[list {$lruOne} {$lruTwo} {$lruThree} {$lruFour}\]"
			
		close $out
	}

	# exit application
	destroy .
}


# procedure for changing current document (called by menu)
proc ChangeWindow {} {

	foreach var { fileName fileName_old codeWin infoWin outWin openFiles high } {		
		variable $var
	}
	
	if {$fileName != $fileName_old} {
		
		# insert current window contents later into corresponding list element
		variable inhalt [$codeWin get 1.0 end]

		# remove last '\n'
		regsub {\n$} $inhalt {} inhalt

		variable test [lsearch -regexp $openFiles "^<$fileName_old>.*"]
		
		if {$test != -1} {

			# insert current window contents into corresponding list element
			set openFiles [lreplace $openFiles $test $test "<$fileName_old>$inhalt"]
		}

		# read and display new window contents
		set test [lsearch -regexp $openFiles "^<$fileName>.*"]
		
		if {$test != -1} {
			
			# filter contents out
			regexp "^<$fileName>(.*)$" "[lindex $openFiles $test]" dummie inhalt
			
			# clear window
			$codeWin delete 1.0 end
			$infoWin delete 1.0 end
			$outWin delete 1.0 end

			# insert current contents
			$codeWin insert end $inhalt
		}
	
		# accustomize helper variable to new name
		set fileName_old $fileName

		wm title .env " iVRS -- $fileName"

		# reactivate syntax highlighting, if been active before.
		if {$high == 1} {
			
			SetHighlight 1.0 end
		}
	}
}


# procedure for changing current font size (called by menu)
proc ChangeSize {} {

	variable size 
	
	# for all text windows
	foreach var { codeWin outWin infoWin } {

		# make variables known
		variable $var

		# var contains name for real variable; associate varUp with this one
		upvar devADS::$var varUp

		# change font size
		$varUp configure -font "courier $size"
	}
}


# procedure for changing syntax highlighting composition (switch components on/off)
proc ChangeHighlight {name} {

	foreach var { high checkVrs checkTcl checkNav checkCom checkEnu checkTk checkVari } {
		
		# make variable known
		variable $var
	}
	
	if {$high == 1} {
	
		# 'name' contains corresponding tag name of changed variable; association herewith
		upvar devADS::check$name check
	
		# if an option was switched on
		if {$check == 0} {
		
			variable codeWin
			# remove chars bound to corresponding tag
			$codeWin tag remove $name 1.0 end
			
		} else {
			
			# save variable contents
			foreach var { Vrs Tcl Nav Com Enu Vari Tk } {
				
				# associate 'devADS::check$var' with variable 'check' (because of double dollar sign)
				upvar devADS::check$var check
				
				# create rescue variable
				variable save$var $check
			}
			
			# remove $name from list 
			variable liste ""
			regsub "$name " {Vrs Tcl Nav Com Enu Tk Vari } {} liste
			
			# highlight only new aspect 
			foreach var $liste {
					
				# reset temporarily
				set check$var 0
			}

			# call syntax highlighting procedure 
			SetHighlight 1.0 end
				
			# reset variable contents 
			foreach var $liste {
					
				# associate 'devADS::check$var' with variable 'check' (because of double dollar sign)
				upvar devADS::save$var save
					
				# restore original values
				set check$var $save
			}
		}
	}
}


# procedure for display of all available class methods, constructors and enums of class '$wort'
# called by <Ctrl-Tab>-Binding
proc OutputClassInfo {wort} {

	variable infoWin
	
	$infoWin delete 1.0 end
	$infoWin tag remove info 1.0 end

	variable constructors [VRS info ctors $wort]

	if {$constructors != ""} {

		$infoWin insert end "$wort constructors:"
		$infoWin tag add info "insert linestart" "insert lineend"
		$infoWin insert end "\n\n"
		foreach {name type alias} $constructors {
			
			$infoWin insert end "argument datatypes: \n"
			$infoWin insert end "{$type}\nargument names:\n"
			$infoWin insert end "{$alias}\n\n"
		}
	}

	variable methods [VRS info methods $wort]
	$infoWin insert end "\n$wort methods:"
	$infoWin tag add info "insert linestart" "insert lineend"
	$infoWin insert end "\nreturnValue method {argument datatypes} {argument names}"
	$infoWin tag add info "insert linestart" "insert lineend"
	$infoWin insert end "\n\n$methods"

	variable statics [VRS info statics $wort]

	if {$statics != ""} { 

		$infoWin insert end "\n\n$wort static methods:"
		$infoWin tag add info "insert linestart" "insert lineend"
		$infoWin insert end "\nreturnValue method {argument datatypes} {argument names}"
		$infoWin tag add info "insert linestart" "insert lineend"
		$infoWin insert end "\n\n$statics"
	}

	variable enums ""
	catch {
		set enums [VRS info enums $wort]
		$infoWin insert end "\n\n$wort enums:"
		$infoWin tag add info "insert linestart" "insert lineend"
		$infoWin insert end "\n\n$enums"
	}
			
			
	# pop up menu
	if {[winfo exists .popup]} { destroy .popup }
	menu .popup -tearoff false
		
		if {$constructors != ""} {
		 
			.popup add cascade -label "$wort constructors" -menu .popup.const
			menu .popup.const -tearoff false
		}

		.popup add cascade -label "$wort methods" -menu .popup.meth
		
		if {$statics != ""} { 

			.popup add cascade -label "$wort static methods" -menu .popup.stat
			menu .popup.stat -tearoff false
		}

		if {$enums != ""} {
			.popup add cascade -label "$wort enums" -menu .popup.enum
			menu .popup.enum -tearoff false
		}

	menu .popup.meth -tearoff false
							
	foreach {name type alias def} $constructors {
		.popup.const add command -label "argument datatypes:" -command "destroy .popup"
		.popup.const add command -label "{$type}" -command "devADS::Insert {{$type}} ; destroy .popup"
		.popup.const add command -label "argument names:" -command "destroy .popup"
		.popup.const add command -label "{$alias}" -command "devADS::Insert {{$alias}} ; destroy .popup"
		.popup.const add separator
	}
			
	foreach {return name varname def} $methods {
		.popup.meth add command -label "$return $name {$varname} {$def}" -command "devADS::Insert $name ; destroy .popup"
	}

	foreach {return name varname def} $statics {
		.popup.stat add command -label "$return $name {$varname} {$def}" -command "devADS::Insert $name ; destroy .popup"
	}

	if {$enums != ""} {
		foreach {thing} $enums {
			.popup.enum add command -label $thing -command "devADS::Insert $thing ; destroy .popup"
		}
	}

	tk_popup .popup [winfo pointerx .] [winfo pointery .]
} 


# procedure for opening recently used files (called by menu)
proc OpenLRU {file_} {

	if {[file exists "$file_"]} {

		variable codeWin 
			
		# if window previously write protected
		$codeWin configure -state normal
		
		# call general file opening procedure
		Open_inGeneral "$file_"
	
	} else {

		# display error message
		if {[winfo exists .fileNotExists]} {

			wm deiconify .fileNotExists
	
		} else {

			toplevel .fileNotExists
			wm title .fileNotExists " file does not exist ..." 
			wm geometry .fileNotExists 300x110+140+220
			wm resizable .fileNotExists 0 0
		
			# interface surface design
			
			frame .fileNotExists.oben -borderwidth 15
			frame .fileNotExists.unten
			pack .fileNotExists.oben .fileNotExists.unten -side top

			label .fileNotExists.oben.info -text "Error:	The file you chose doesn't exist."
			button .fileNotExists.unten.ok -text "OK" -command "destroy .fileNotExists" -width 20 -height 2
			pack .fileNotExists.oben.info .fileNotExists.unten.ok -side bottom

			# concentrate on information window and wait for its destruction
			grab .fileNotExists
			tkwait window .fileNotExists
		}
	}
}


# general file opening procedure (called from procs 'OpenLRU' and 'OpenFile')
proc Open_inGeneral {file_} {

	foreach var { fileName lruOne lruTwo lruThree lruFour winMenu lruMenu fileList openFiles } {
		variable $var
	}
	
	if [catch {$winMenu index "$file_"} index] {
	
		set fileName $file_
		
		if [catch {open $fileName r} input] {
			
			tk_messageBox -type ok -message "an error occurred while opening $fileName " -icon error

		} else {
		
			variable contents [read $input]
			close $input
			
			# enter new element into list
			set openFiles [linsert $openFiles 0 "<$fileName>$contents"]
			
			# call procedure for window change
			ChangeWindow

			wm title .env " iVRS -- $fileName"

			# make entry into 'window'
			$winMenu add radio -label "$fileName" -variable devADS::fileName -value $devADS::fileName -command "devADS::ChangeWindow"

			# remove old menu entries
			$lruMenu delete 0 3
						
			variable test [lsearch $fileList "$fileName"]

			if {$test != -1} {
							
				# remove old entry
				set fileList [lreplace $fileList $test $test]
			}

			# prepend new file name 
			set fileList [linsert $fileList 0 "$fileName"]

			# refresh entry order
			set lruOne [lindex $fileList 0]
			set lruTwo [lindex $fileList 1]
			set lruThree [lindex $fileList 2]
			set lruFour [lindex $fileList 3]

			# insert refreshed entries
			$lruMenu add command -label "1   $lruOne" -command "devADS::OpenLRU {$lruOne}"
			$lruMenu add command -label "2   $lruTwo" -command "devADS::OpenLRU {$lruTwo}"
			$lruMenu add command -label "3   $lruThree" -command "devADS::OpenLRU {$lruThree}"
			$lruMenu add command -label "4   $lruFour" -command "devADS::OpenLRU {$lruFour}"
		}
		
	} else {
		
		# create error message
		if {[winfo exists .alreadyOpen]} {

			wm deiconify .alreadyOpen
	
		} else {

			toplevel .alreadyOpen
			wm title .alreadyOpen " file is already open ..." 
			wm geometry .alreadyOpen 300x110+140+220
			wm resizable .alreadyOpen 0 0

			# 'already open' window surface design
			
			frame .alreadyOpen.oben -borderwidth 15
			frame .alreadyOpen.unten
			pack .alreadyOpen.oben .alreadyOpen.unten -side top

			label .alreadyOpen.oben.info -text "Error:	The file you chose is already open."
			button .alreadyOpen.unten.ok -text "OK" -command "destroy .alreadyOpen" -width 20 -height 2
			pack .alreadyOpen.oben.info .alreadyOpen.unten.ok -side bottom
			
			# concentrate on information window and wait for its destruction
			grab .alreadyOpen
			tkwait window .alreadyOpen			
		}
	}
}


# procedure for display of the iVRS HTML-help (called via menu)
proc ViewiVRSHelp {} {

	# show file in default browser
	catch {eval exec [auto_execok start] [list ../www/index.html] &}
}



# procedure for displaying HTML documentation of the IDE in browser (called via menu)
proc ViewIDEHelp {} {

	# show file in default browser
	catch {eval exec [auto_execok start] [list index.html] &}
}


# procedure for displaying the about-Box (called via menu)
proc About {} {

	# create About-Box
	if {[winfo exists .about]} {

		wm deiconify .about
	
	} else {

		toplevel .about
		wm title .about " about ..." 
		wm geometry .about 300x130+140+220
		wm resizable .about 0 0

		# window design of 'about' alert
			
		frame .about.oben -borderwidth 15
		frame .about.unten
		pack .about.oben .about.unten -side top

		label .about.oben.info -text "iVRS IDE \n\nwritten by: Anne Rozinat"
		button .about.unten.ok -text "OK" -command "wm withdraw .about" -width 20 -height 2
		pack .about.oben.info .about.unten.ok -side bottom
	}
}



# procedure for displaying class information (called by Tree-Widget) [by Christian]
set windowcounter 0

proc ShowIvrsClassInfo { classname } {	
	
	if {[catch {VRS info classname $classname}]} {	return }

	# increase window counter and create new top level
	variable windowcounter
	set windowname .iVRSmethodInfo$windowcounter
	incr windowcounter

	toplevel $windowname
	

	wm title $windowname "Information about iVRS class \"$classname\":"


	frame $windowname.upper -borderwidth 5
	pack $windowname.upper -side top -expand true -fill both

	text $windowname.upper.info -width 70 -height 30 -wrap word -yscrollcommand "$windowname.upper.infoScroll set"
	scrollbar $windowname.upper.infoScroll -orient vert -command "$windowname.upper.info yview"
	
	pack $windowname.upper.info -side left -fill both -expand true 
	pack $windowname.upper.infoScroll -side right -fill y

	
	
	frame $windowname.lower -borderwidth 5
	pack $windowname.lower -side top -expand true -fill both

	button $windowname.lower.close -text "close this window" -command "destroy $windowname" -width 15 -height 1
	pack $windowname.lower.close -side left -anchor s


	$windowname.upper.info tag configure bluecode -foreground SteelBlue4 -font *-courier-bold-r-*-14-*
	$windowname.upper.info tag bind bluecode <ButtonPress> "devADS::ShowIvrsClassInfo \[$windowname.upper.info get \"@%x,%y wordstart\" \"@%x,%y wordend\"\]"
	$windowname.upper.info tag configure greencode -foreground green4 -font *-courier-bold-r-*-14-*
	$windowname.upper.info tag configure blackcode -foreground black -font *-courier-bold-r-*-14-*
	$windowname.upper.info tag configure headline1 -foreground black -font *-helvetica-bold-r-*-16-*
	$windowname.upper.info tag configure headline2 -foreground black -font *-helvetica-medium-r-*-14-* -underline true
	
	
	# start writing class info
	$windowname.upper.info insert 1.0 "\nClass \"$classname\":\n\n" headline1
	
	# write enums
	$windowname.upper.info insert end "Enumerations:\n" headline2
	if { [llength [VRS info enums $classname]] == 0 } {
		$windowname.upper.info insert end "no enumerations.\n" greencode	
	}
	set last [expr [llength [VRS info enums $classname]] - 1] 
	set pos 0
	foreach enum [VRS info enums $classname] {
		$windowname.upper.info insert end $enum greencode
		if { $pos != $last } { $windowname.upper.info insert end ", " blackcode }
		incr pos
	}
	$windowname.upper.info insert end "\n"
	
	# write constructors
	$windowname.upper.info insert end "\nConstructors:\n" headline2

	if { [llength [VRS info ctors $classname]] == 0 } {
		$windowname.upper.info insert end "no constructors.\n" greencode	
	}
	
	foreach {ctorname argtypes argnames defs } [VRS info ctors $classname] {
		$windowname.upper.info insert end "$ctorname \{" blackcode
		set last [expr [llength $argtypes] - 1] 
		set pos 0
		foreach argtype $argtypes argname $argnames {
			$windowname.upper.info insert end "$argtype " bluecode
			$windowname.upper.info insert end $argname blackcode
			if { $pos != $last } { $windowname.upper.info insert end ", " blackcode }
			incr pos	
		}
		$windowname.upper.info insert end "\}\n" blackcode
	}

	
	# write methods
	$windowname.upper.info insert end "\nMethods:\n" headline2

	if { [llength [VRS info methods $classname]] == 0 } {
		$windowname.upper.info insert end "no methods.\n" greencode	
	}

	foreach {return name argtypes argnames defs} [VRS info methods $classname] {
		$windowname.upper.info insert end $return bluecode
		$windowname.upper.info insert end " $name \{" blackcode
		set last [expr [llength $argtypes] - 1] 
		set pos 0
		foreach argtype $argtypes argname $argnames {
			$windowname.upper.info insert end "$argtype " bluecode
			$windowname.upper.info insert end $argname blackcode
			if { $pos != $last } { $windowname.upper.info insert end ", " blackcode }
			incr pos	
		}
		$windowname.upper.info insert end "\}\n" blackcode

	}


	# write static methods
	$windowname.upper.info insert end "\nStatic Methods:\n" headline2

	if { [llength [VRS info statics $classname]] == 0 } {
		$windowname.upper.info insert end "no static methods.\n" greencode	
	}

	foreach {return name argtypes argnames defs} [VRS info statics $classname] {
		$windowname.upper.info insert end $return bluecode
		$windowname.upper.info insert end " $name \{" blackcode
		set last [expr [llength $argtypes] - 1] 
		set pos 0
		foreach argtype $argtypes argname $argnames {
			$windowname.upper.info insert end "$argtype " bluecode
			$windowname.upper.info insert end $argname blackcode
			if { $pos != $last } { $windowname.upper.info insert end ", " blackcode }
			incr pos	
		}
		$windowname.upper.info insert end "\}\n" blackcode

	}


	# write parent class
	$windowname.upper.info insert end "\nParent Class:\n" headline2
	if { [VRS info parentclass $classname] == "void" } {
		$windowname.upper.info insert end "no parent class.\n" greencode	
	
	} else {
	
		# insert classname
		$windowname.upper.info insert end "[VRS info parentclass $classname]" bluecode
		$windowname.upper.info insert end "\n"
	}


	# write child classes
	$windowname.upper.info insert end "\nChild Classes:\n" headline2
	if { [llength [VRS info childclasses $classname]] == 0 } {
		$windowname.upper.info insert end "no child classes.\n" greencode	
	}
	set last [expr [llength [VRS info childclasses $classname]] - 1] 
	set pos 0
	foreach enum [VRS info childclasses $classname] {
		$windowname.upper.info insert end $enum bluecode
		if { $pos != $last } { $windowname.upper.info insert end ", " blackcode }
		incr pos
	}
	$windowname.upper.info insert end "\n"
}


# procedure, for showing the TkCon console (but only in application interpreter)
proc ShowConsole {} {

	if {[interp exists app]} {
		
		if { [app eval {winfo exists .}] == "1" } {
		
			app eval source [file join .. tkcon.tcl]
		}
			
	} else {
		
		if {[winfo exists .conError]} {

			wm deiconify .conError
	
		} else {
		
			toplevel .conError
			wm title .conError " error " 
			wm geometry .conError 300x130+140+220
			wm resizable .conError 0 0

			# window design of the 'about' window/alert
			
			frame .conError.oben -borderwidth 15
			frame .conError.unten
			pack .conError.oben .conError.unten -side top

			label .conError.oben.info -text "Cannot show console without running application. \nRun an empty application if you want!"
			button .conError.unten.ok -text "OK" -command "wm withdraw .conError" -width 20 -height 2
			pack .conError.oben.info .conError.unten.ok -side bottom
		}
	}
}	



# procedure for recursively opening all parent nodes of a node in tree (make visible)
proc MakeVisible { knoten } {

	variable li
	
	# detect parent node
	variable vater [$li.canvas.tree parent $knoten]

	if {$vater != "root"} {
		
		# open parent node
		$li.canvas.tree opentree $vater

		# close all child nodes except 'knoten'
		foreach nod [$li.canvas.tree nodes $vater] {
			
			if {$nod != $knoten} {

				# close subtrees
				$li.canvas.tree closetree $nod
			}
		}

		# make parent node visible
		MakeVisible $vater
	}	
}





# overload window closing command
wm protocol .env WM_DELETE_WINDOW {devADS::ExitApp}


# close namespace
}


wm deiconify .env 









