# load package
source initvrs.tcl


toplevel .opengl
set canvas [new TclCanvas  .opengl.main  320  400 GLCanvas::RGBADDS]    
pack .opengl.main -expand yes -fill both

set config [new SceneConfiguration]
$config append [new BackgroundGL {0.2 0.2 0.2 1.0} {0.9 0.9 0.9 1.0} BackgroundGL::TopToBottom]

set distantlight [new DistantLight {0 0 -2} {0.5 0.5 0.5 1.0}]
set redlight [new PointLight {0 0 -2} {1 0.2 0.2 1.0}]

# create shadowed background
set back [new SceneThing]

set shadowed [new Shadowed $distantlight]
set shadowSwitch [new ShadowedSwitch]
$shadowSwitch add $shadowed true

$back append $shadowed
$back append $shadowSwitch
$back append [new Translation 0 0 2]
$back append [new ColorAttribute 0.8 0.8 0.8]
$back append [new Material 0.6 1.0 0.5]
$back append [new Cache [new Box {-1.8 -1.8 0} {1.8 1.8 -0.2}]]
$back append [new Rotation {-1 0 0}  {0 0 0} 90]
$back append [new Cache [new Sphere 1 {-0.6 0 0.6} 0]]
$back append [new Cache [new Sphere 1 {-0.6 0 -0.6} 0]]
$back append [new Cache [new Sphere 1 {0.6 0 0.6} 0]]
$back append [new Cache [new Sphere 1 {0.6 0 -0.6} 0]]

# create shadowcasting foreground
set front [new SceneThing]

set cast [new ShadowCaster $distantlight]
set shadowCast [new ShadowCasterSwitch]
$shadowCast add $cast true

$front append $cast
$front append $shadowCast
$front append [new Material 0.6 1.0 0.3]
$front append [new ColorAttribute 0.5 1.0 0.5 ]
$front append [new Cache [set torus [new Torus ]]]

# create main scene graph
set thing [new SceneThing]
$thing append [new ShadowTechniqueGL]
$thing append $distantlight
$thing append [new ConstantDetail 25 25]
$thing append $back
$thing append $front

# create permanent installations
set permanentLight [new SceneThing]
$permanentLight append [new DistantLight {0 0 -2} {0.3 0.5 0.7 1.0}]
    
# create camera
set camera [new SceneThing]
$camera append [set cam [new Camera [new Perspective 60 0.1 10.0] [new LookAt {0 0 -2} {0 0 0}]]]

# all geometry graphs
$canvas append $config
$canvas append $camera
$canvas append $permanentLight
$canvas append $thing

# create and connect behaviors
$canvas append [new TrackBall $thing]
$canvas append [new TrackBall $front InputEvent::MouseButton3]


# check for pbuffer
if {$tcl_platform(platform) == "windows"} {
    update
    if {![regexp WGL_ARB_pbuffer [$canvas glInfo]]} {
	    tk_messageBox -message "Sorry, need WGL_ARB_pbuffer extension to run." -icon error
	    destroy .opengl
	    VRS cleanup
    }
} 
