# load package if isn't loaded
package require VRS

# camera settings
set mview [new SceneThing]
$mview append [new Camera {0 3 2} {0 0 0}  60]

# main thing
set mscene [new SceneThing $mview]

# thing with floor
set menviron [new SceneThing $mscene]

# lightsources 
$mscene append [new DistantLight { -0.1 0 -1 } {1 1 1 1} 1.0 0.9]
$mscene append [new PointLight { 1 2 1 } {1 1 1 1} 1.0  0.8]

# thing with snowman
set msnowman [new SceneThing $mscene]

set mbody_ [new SceneThing $msnowman]
$mbody_ append [new ColorAttribute 1 1 1 ]
$mbody_ append [new Scaling {1 0.9 1 }]
$mbody_ append [new FaceStyle FaceStyle::Filled FaceStyle::Culled]
$mbody_ append [new Sphere 0.5  {0 0.43 0}]
$mbody_ append [new Sphere 0.35 {0 1.1 0}]
$mbody_ append [new Sphere 0.25 {0 1.5 0}]
$mbody_ append [new ColorAttribute { 0.7 0.7 1.0 1 }]
$mbody_ append [new Box {-1 -0.08 -1 } { 1 0 1 }]

set mface [new SceneThing $msnowman]
set meyes_ [new SceneThing $mface]
$meyes_ append [new ColorAttribute 0.1 0.1 1.0]
$meyes_ append [new Sphere 0.04  {-0.1 1.5 0.12}]
$meyes_ append [new Sphere 0.04  { 0.1 1.5 0.12}]

set mnose_ [new SceneThing $mface]
$mnose_ append [new ColorAttribute 1 0.5 0]
$mnose_ append [new Cone {0 1.55 0.2} {0 1.35 0.22} 0.00  0.04]

set mmouth_ [ new SceneThing $mface]
$mmouth_ append [new ColorAttribute 0.9 0.9 0]
$mmouth_ append [new Cylinder {-0.12 1.33 0.21} {.12 1.33 0.21}  0.05]

# store a few shapes so we can identify them later
[$mbody_ object 3]  setObjectName  "lower part" 
[$mbody_ object 4]  setObjectName  "middle part"
[$mbody_ object 5]  setObjectName  "upper part"
[$mbody_ object 7]  setObjectName  "box"
[$meyes_ object 1]  setObjectName  "left eye"
[$meyes_ object 2]  setObjectName  "right eye"
[$mnose_ object 1]  setObjectName  "nose"
[$mmouth_ object 1]  setObjectName  "mouth"

 
# MENUS

frame .mbar -border 1 -relief raised
menubutton .mbar.file -text File -menu .mbar.file.m

menu .mbar.file.m
.mbar.file.m add command -label "Quit" -command "destroy .; VRS cleanup"
pack .mbar.file -side left
pack .mbar -fill x

# build a 3D canvas
set canvas [ new TclCanvas .toto 256  256 ]
pack .toto

# connect geometry graphs to the canvas
$canvas append $mview
$canvas append [new TrackBall $mscene]


#
# STATUS AREA
#
frame .state -relief sunken -border 2
label .state.msg -textvariable statustext -justify left -anchor w 
pack .state -side bottom -fill x  
pack .state.msg -side left -fill x 


# ray intersection
bind . <ButtonPress-3> {
	if {[info exists rayengine] == 0 || [info commands $rayengine] == ""} {
		set rayengine [new Engine]
		set rayrequest [ new RayRequest RayRequestTechnique::Nearest NULL { 0.0 0.0 0.0    0.0 0.0 -1.0 } RayRequestTechnique::Clip]	
		$canvas install $rayengine
	}
    
	# calculate clip coordinates
	set w [winfo width %W]
    set h [winfo height %W]
    set x [expr (2.0*%x)/$w - 1.0]
    set y [expr (-2.0*%y)/$h + 1.0]

	# do ray test		
	$rayrequest setRay "$x $y 0   0 0 1" 
	$canvas eval $rayengine $rayrequest    

	# print hit objects
	set hits [$rayrequest results]	
	for { set i 0} {$i < $hits } {incr i} {		
		set iinfo [$rayrequest result $i]
		set shape [$iinfo getShape]	
		set statustext [$shape getObjectName]        
		#delete $iinfo
	}
}

