# load package if isn't loaded
package require VRS

# make view
set view [new SceneThing]
set projection  [new Perspective  60  0.1  10.0]
set orientation [new LookAt { 0 0 -4 }  { 0 0 0  }]
$view append [new Camera $projection $orientation]

# main thing
set mainThing [new SceneThing $view]  
$mainThing append   [new DistantLight  {1 1 1}]
$mainThing append   [new PointLight  {0 1 -2} {1 1 1 1} 0.5 0.8]

# make VRS objects
set texture1_   [ new ImageTexture2DGL [ObjectLoader readFile pics/vrs.ppm]]
$texture1_ setBorderColor { 1  0  0 1}
$texture1_ setMinFilter TextureGL::NEAREST

set texture2_   [ new ImageTexture2DGL [ObjectLoader readFile pics/cafe.rgb]]
$texture2_ setBorderColor { 1  0  0 1}
$texture2_ setMinFilter TextureGL::NEAREST

set texture3_   [ new ImageTexture2DGL [ObjectLoader readFile pics/tree.rgb]]
$texture3_ setBorderColor { 1  0  0 1}
$texture3_ setMinFilter TextureGL::NEAREST

set texture4_   [ new ImageTexture2DGL [ObjectLoader readFile pics/cafe.rgb]]
$texture4_ setBorderColor { 0  1  0 1}
$texture4_ setMinFilter TextureGL::NEAREST
   

set box [new Box {-1 -1 -1}  {1 1 1}]
set torus [new Torus]
set sphere [new Sphere]
set texenv [new TexEnvGL TexEnvGL::Modulate { 1  0.7  0.5 1}]
set texGen_ [new TexGenGL TexGenGL::Object  1  0  0  0  0  1  0  0]
set currentShape_  $box
set currentTexture_ $texture1_

# make 3D canvas

set canvas [new TclCanvas  .main  320  400 ]

# make core part of scene graph
set thing_ [new SceneAdjuster [new Box {-1 -1 -1} { 1 1 1 } ] true {0 0 0} true]


# use static method of ColorBufferGL tp create alphatest
$thing_ append [ColorBufferGL newAlphaTest true GL_GREATER 0]

# use normal ctors
$thing_ append [new ConstantDetail  32]
$thing_ append [new ColorAttribute  {0.7 0.7 1 1}]
$thing_ append [new Material  0.6  1.0  0.0]
$thing_ append [new FaceStyle  FaceStyle::Filled  FaceStyle::Culled]
$thing_ append $texGen_
$thing_ append $texenv  
$thing_ append $currentTexture_  
$thing_ append $currentShape_ 
$mainThing append $thing_      

set config [new SceneConfiguration]
$config append [new BackgroundGL {0.2 0.2 0.2 1.0} {0.9 0.9 0.9 1.0} BackgroundGL::TopToBottom]


# connect geometry graphs and behavior graphs
$canvas append $config
$canvas append $view  
$canvas append [new TrackBall  $mainThing ]

# Menu
frame .mbar -border 1 -relief raised
pack .mbar -fill x

foreach i {File Shape Combine MinFilter MagFilter WrapS WrapT\
		   Anisotropic TexGen Matrix Textures} {
	menubutton .mbar.[string tolower $i] -text $i -menu .mbar.[string tolower $i].m
	menu .mbar.[string tolower $i].m	
	pack .mbar.[string tolower $i] -side left
	bind .mbar.[string tolower $i] <ButtonRelease-1> { $canvas redisplay }
}

.mbar.file.m add command -label "Quit" \
	-command { destroy .; VRS cleanup }


foreach i {BOX TORUS SPHERE} {
   .mbar.shape.m add command -label $i \
		-command "\
			$thing_ replace \$currentShape_ \$[string tolower $i] ;\
			set currentShape_ \$[string tolower $i]"
}

foreach i {BLEND MODULATE DECAL REPLACE ADD } {
   .mbar.combine.m add command -label $i \
		-command "\$texenv setCombineMode TexEnvGL::$i"
}

foreach i {NEAREST LINEAR MIPMAP} {
   .mbar.minfilter.m add command -label $i \
		-command "\$currentTexture_ setMinFilter TextureGL::$i"
}

foreach i {NEAREST LINEAR } {
   .mbar.magfilter.m add command -label $i \
		-command "\$currentTexture_ setMagFilter TextureGL::$i"
}

foreach i {1 2 4 8} {
   .mbar.anisotropic.m add command -label $i \
			-command "\$currentTexture_ setAnisotropic $i"
}

foreach i {CLAMP REPEAT CLAMPTOEDGE CLAMPTOBORDER MIRROREDREPEAT} {
   .mbar.wraps.m add command -label $i \
			-command "\$currentTexture_ setWrapS TextureGL::$i"
}
    
foreach i {CLAMP REPEAT CLAMPTOEDGE CLAMPTOBORDER MIRROREDREPEAT} {
   .mbar.wrapt.m add command -label $i \
			-command "\$currentTexture_ setWrapT TextureGL::$i"
}


set matrix	 "{8 0 0 0		0 8 0 0		0 0 1 0		1 1 1 1}"
set identity "{1 0 0 0		0 1 0 0		0 0 1 0		1 1 1 1}"

.mbar.matrix.m add command -label true \
	-command " \$currentTexture_ setMatrix $matrix "
.mbar.matrix.m add command -label false \
	-command " \$currentTexture_ setMatrix $identity "

foreach i {OFF OBJECT EYE SPHERICAL REFLECTIONMAP NORMALMAP} {
   .mbar.texgen.m add command -label $i \
		-command "$texGen_ setGenMode TexGenGL::$i"			
}

foreach i {1 2 3} {
   .mbar.textures.m add command -label Texture$i \
		-command "$thing_ replace \$currentTexture_ \$texture$i\_ ; \
				  set currentTexture_ \$texture$i\_"   
}


#
# CANVAS
#
#.main configure -width 200 -height 200
pack .main -fill both -expand yes

#
# STATUS AREA
#
frame .state -relief sunken -border 2
label .state.msg -text "iVRS Sample Application" -justify left -anchor w 
pack .state -side bottom -fill x  
pack .state.msg -side left -fill x 





