
namespace eval db {

	# statics 
	variable methods 0
	variable static  0
	variable abstractmethods 0
	variable polymorphmethods 0
	
	proc initialize {} {
		global data
		if {[array exists data]} { unset data }
		if {[array exists methodTable]} { unset methodTable }
		set data(allClasses) ""		
		puts "data cleaned ..."
	}

	# save / read array data 
	proc writeData {} {
		global data
		set id [open classinfo.txt w]
		puts $id [array get data]
		close $id
	}
	proc readData {} {		
		global data		
		if {[file exists classinfo.txt]} {
			set id [open classinfo.txt r]	
			if {[array exists data]} { unset data }
			array set data [read $id]
			close $id
			return 1
		}
		return 0
	}

	#
	# class 
	#
	proc beginClass { classname } {
		global data
		if {[lsearch -exact $data(allClasses) $classname] == -1 } {
			lappend data(allClasses) $classname			
		}
		set data($classname,parent) ""
		set data($classname,methodtags)	""
	}

	proc endClass { classname } {
	}

	proc eraseClass { classname } {        
        global data
        upvar 0 data(allClasses) classes
        set idx [lsearch -exact $classes $classname]
		if {$idx != -1 } { set data(allClasses) [lreplace $classes $idx $idx]}
		array unset data "$classname,*"		
	}

	proc isClass { classname } {
		global data
		return [info exists data($classname,parent)]
	}

	proc getClasses {} {
		global data
		return $data(allClasses)
	}

	proc hasCtor { classname } {
		global data
		return [info exists data($classname,hasctor)]
	}

	#
	# child
	#

	proc addChild { classname child} {
		global data
		if {[info exists data($classname,childs)]} {
			lappend data($classname,childs) $child
		} else {
			set data($classname,childs) $child
		}
	}

	proc getChilds { classname } {
		global data 
		if {[info exists data($classname,childs)]} {
			return $data($classname,childs)
		} 
		return ""
	}

	proc hasChilds { classname } {
		global data 
		return [info exists data($classname,childs)]
	}


	#
	# parent
	#

	proc setParent { classname parent } {
		global data 
		set data($classname,parent) $parent
		
		if {[string length $parent]} {
			regsub {<.*>} $parent {} parent
			regsub {<|>} $parent {} parent
			addChild [string trim $parent] $classname
		} 
	}


	proc getParent { classname } {
		global data 
		return [string map {% " "} $data($classname,parent)]
	 
	}

	#
	# template
	#

	proc setTemplate { classname template } {
		global data 
		set data($classname,template) $template
	}


	proc getTemplate { classname } {
		global data 
		if {[info exists data($classname,template)]} {
			return [string map {% " "} $data($classname,template)]
		} else {
			return ""
		}
	}

	#
	# timestamp
	#

	proc isUptodate { filename } {
		global data 
		set cur [file mtime $filename]
		set tail [file tail $filename]
		if { [info exists data($tail)] && $cur <= $data($tail) } {
			return 1
		} else {
			set data($tail) $cur
			return 0
		}
	}


	#
	# filename
	#

	proc setFilename { classname filename } {
		global data 
		set data($classname,filename) $filename
	}

	proc getFilename { classname } {
		global data 
		return $data($classname,filename)
	}

	#
	# enum handling
	#

	proc addEnum { classname enumname } {
		global data 
		set data($classname,enum,$enumname) ""		
	}

	proc addEnumElement {classname enumname enumelt } {
		global data 
		if {![info exists data($classname,enum,$enumname)]} {
			error "enumname $enumname not included in class $classname"			
		} 
		
		lappend data($classname,enum,$enumname) $enumelt 
	}

	proc hasEnums { classname } {
		global data
		return [expr [llength [array get data "$classname,enum,*"]] != 0]
	}

	proc getEnums { classname } {
		global data
		set res [array get data "$classname,enum,*"]
		regsub -all {\S+,enum,} $res {} res
		return $res
	}



	#
	# method handling
	#
	proc beginMethodParse { classname } {
		global data tmp	
		array unset tmp

		set tmp(classname) $classname  
		set tmp(name) ""
		set tmp(returnType) ""
		set tmp(args) ""
		set tmp(vars) ""
		set tmp(defs) ""
		set tmp(flags) ""
		set tmp(enumnames) ""
		foreach {name vals} [getEnums $classname] {
			lappend tmp(enumnames) $name
		}
	}

	
	proc setMethodName { name } {
		global tmp
		set tmp(name) $name
	}

	proc setMethodReturnType { returnType } {
		global tmp
		foreach e $tmp(enumnames) {
			if {[regexp $e $returnType] && ![regexp $tmp(classname)\:: $returnType]} {
			    regsub -all $e $returnType $tmp(classname)\::$e returnType
			}
		}
		set tmp(returnType) $returnType
	}

	proc addMethodArgument {arg var def} {
		global tmp data
		if { $var == "" } { set var noname }

		# expand enumeration types with class name
		foreach e $tmp(enumnames) {
			if {[regexp $e $arg] && ![regexp $tmp(classname)\:: $arg]} {
			    regsub -all $e $arg $tmp(classname)\::$e arg
			}
		}

		# defaults 
		lappend tmp(args) [string trim $arg]
		lappend tmp(vars) [string trim $var]		
        lappend tmp(defs) [string trim $def]
	}

	proc addMethodFlag { flag } {
		global tmp 		
		lappend tmp(flags) $flag
	}

	proc generateNumTag { tagref } {
		upvar $tagref tag
		set res 0
		set pos 2
		foreach {a b} "[split $tag {}] [split $tag {}]" {
			incr res [expr [scan $a %c] * ($pos+1)]
			incr res [expr [scan $b %c] * ($pos-1)]
			incr pos
		}
		return $res
	}
	proc endMethodParse { classname  } {
		global data tmp				
		
		
		# expand enumeration types with class name
		if {[lsearch -exact $tmp(enumnames) $tmp(returnType)] != -1 } {
			set tmp(returnType) $classname\::$tmp(returnType)
		}


		# code methodname
		#regsub -all {\s+|%|<|>|\:\:} "_$tmp(name) $tmp(args)" {_} tag
		#regsub -all {\*} $tag {P} tag
		#if {[regexp {const} $tmp(flags)]} {	append tag const }
		
		set tag $tmp(name) 
		if {[regexp {const} $tmp(flags)]} {	append tag const }
		set num1 [generateNumTag tag]
		set num2 [generateNumTag tmp(args)]
		set tag "m_$num1\_$num2"

		lappend data($classname,methodtags) $tag
		
		if {[regexp ctor $tmp(flags)]} {
			set data($classname,hasctor) 1
		}

		set data($classname,method,$tag,name) $tmp(name)
		set data($classname,method,$tag,returnType) $tmp(returnType)
		set data($classname,method,$tag,args) $tmp(args)
		set data($classname,method,$tag,vars) $tmp(vars)
		set data($classname,method,$tag,defs) $tmp(defs)
		set data($classname,method,$tag,flags) $tmp(flags)
		
		
		variable methods
		variable static
		variable abstractmethods
		
		incr methods
		if {[regexp static $tmp(flags)]} { incr static }
		if {[regexp abstract $tmp(flags)]} { 
			incr abstractmethods 
		}	
	}

	proc getMethodTags { classname } {
		global data
		return $data($classname,methodtags)
	}

	proc getMethodArgs {classname tag } {
		global data
		return $data($classname,method,$tag,args)
	}
	proc getMethodVars {classname tag } {
		global data
		return $data($classname,method,$tag,vars)		
	}
	proc getMethodDefs {classname tag } {
		global data
        set res $data($classname,method,$tag,defs)
        if {[regsub -all {\\\"} $res {"} res]} {
            puts $res
        }
		return $res
	}
	proc getMethodName {classname tag } {
		global data
		if { [regexp ctor $data($classname,method,$tag,flags)] } {
			return $classname
		} else {
			return $data($classname,method,$tag,name) 
		}
	}
	proc getMethodReturnType {classname tag } {
		global data
		return $data($classname,method,$tag,returnType)
	}
	proc getMethodFlags {classname tag } {
		global data
		return $data($classname,method,$tag,flags)
	}

	#
	# dependencies
	#

	proc getDependingClasses { classname } {
		global data
		set dep [array get data "$classname,method,*,returnType"]
		lappend dep [array get data "$classname,method,*,args"]	
		regsub -all {\w+,method,\w+,args|\w+,method,\w+,returnType} $dep "" dep		
		lappend dep $classname [getParent $classname]		
		
		# HACK BEGIN
		regsub -all {ImageRasterData} $dep {Color ImageRasterData} dep
		# HACK END

		regsub -all {<|>} $dep { } dep		
		regsub -all {%|&|SO} $dep {} dep
		regsub -all {::(.*?) } $dep { } dep
		regsub -all {\\\}} $dep {} dep
		regsub -all {|\{|\}|\*} $dep {} dep		
		regsub -all {\s+} $dep { } dep
		set dep [lsort -unique $dep]
		
		set result ""
		foreach d $dep {
			if {[info exists data($d,filename)]} {
				lappend result $d
			}
		}
		return [lsort -unique $result]
	}	


	# generate method table
	proc calcMethodTable { classname } {
		global data methodTable

		if {![db::isClass $classname] } {
			puts stderr "WARNING: $classname"
			return 
		}

		if {[info exists methodTable($classname)] != 0} {
			return 
		}	
				
		if {[info exists data($classname,parent)] && $data($classname,parent) != ""} {
			
			set parent $data($classname,parent)
			regsub {<.*>} $parent {} parent
			regsub -all {<|>} $parent {} parent
			set parent [string trim $parent]
			
			calcMethodTable $parent
			if {[info exists methodTable($parent)] != 0} {
				array set tmpArray $methodTable($parent)
			}

		} else {
			array unset tmpArray 
		}

		variable polymorphmethods
		foreach tag [getMethodTags $classname]  {
			set flags [getMethodFlags $classname $tag ]
			if {[regexp {ctor} $flags]} { continue }

			if {[info exists tmpArray($tag)]} {
				incr polymorphmethods
			}

			if {[regexp {abstract} $flags]} {
                #puts "$classname [getMethodName $classname $tag]"
				set tmpArray($tag) NOCLASS
			} else {
				set tmpArray($tag) $classname
			}
		}

		set methodTable($classname) [array get tmpArray]
	}

	proc isAbstract { classname } {
		global methodTable
		
		# generate method table
		calcMethodTable $classname

		# if one method is not implemented, class is abstract
		if {[regexp {NOCLASS} $methodTable($classname)] != 0} {
			return 1
		}	
		return 0	
	}

    proc overridesAbstractMethod { classname methodtag } {
		global methodTable
		
        if {[regexp {abstract} [getMethodFlags $classname $methodtag]]} {
            return 0
        }
		
        set cur $classname
        while { $cur != "" } {
            regsub {<.*>} $cur {} cur
			regsub -all {<|>} $cur {} cur
			set cur [string trim $cur]

            array set mt $methodTable($cur)
            if {[info exists mt($methodtag)] && $mt($methodtag) == "NOCLASS"} { return 1 }
            set cur [getParent $cur]
            array unset mt
        }
		
		return 0
	}	
	
	proc printStatics {} {

		global data methodTable
		variable methods
		variable static
		variable abstractmethods 
		variable polymorphmethods
		
		set classes 0
        set templateclasses 0
		set baseclasses 0
		set leafclasses 0
		set abstractclasses ""
        set simpleclasses 0
		set overloaded 0
		
		foreach c $data(allClasses) {
			if {[isAbstract $c]} {
				lappend abstractclasses $c
			}
			if {![hasChilds $c]} {
				if {[getParent $c] != ""} {
					incr leafclasses
				} else {
                    incr simpleclasses
                }
			} else {
				incr baseclasses
			}
			
            if {[getTemplate $c] != ""} {
                incr templateclasses
            }
            foreach {tag class} $methodTable($c) {
                if {![regexp {m_(\d+)_} $tag match t]} {
                    error "method tag $tag"
                }                

                if {[info exists count($t)]} {
                    incr count($t)
                } else {
                    set count($t) 0
                }                 
            }
           
            catch {
                foreach {t num} [array get count] {
                    if {$num > 1} {
                        incr overloaded $num
                    }
                }
                unset count            
            }

			incr classes
		}

        set abstractclassesLen [llength $abstractclasses]
		puts "-----------------------------------------------"
		foreach t { classes templateclasses baseclasses leafclasses simpleclasses abstractclassesLen \
                    methods static abstractmethods overloaded \
                    polymorphmethods abstractclasses } {
			puts "$t parsed: [set $t]"		
		}
	}

}

