############################################################
# procedure for writing method signature including prefix
############################################################

proc writeMethodSignatur { classname file prefix tag } {
	set pos 0
	set args [db::getMethodArgs $classname $tag]
	set last [expr [llength $args] - 1]
	set call ""

	foreach type $args def [db::getMethodDefs $classname $tag] {
		# template and unsigned hack backwards
		regsub -all {%} $type { } type
		regsub -all {%} $def { } def

		# care for OpenGL defines
		if {[regexp {GLenum|GLint} $type]} { set gl GL } else { set gl "" }

		if {$type != "ID"} {
			puts -nonewline $file "\t$type arg$pos = "
		} else {
			puts -nonewline $file "\tconst ID& arg$pos = "
		}
		if {$def != ""} { puts -nonewline $file "(argc <= $pos) ? $def : " }
		puts $file "str2type$gl\(argv\[$pos\],\($type*\)0\);"

		append call arg$pos
		if { $pos != $last } { append call "," }

		incr pos
	}
	puts $file "\t$prefix\($call\);"
}


###############################################################
# write to  wrpfiles
###############################################################

proc writeWrapperClass { classname } {
catch {
	global excludeArgs excludeMethods global OptimizeGenerator

	# check if wrapper class is up to date
	set wrpheader "../include/ivrs/gen/[string tolower $classname]wrp.h"
	if {$OptimizeGenerator && [file exists $wrpheader] &&
		[file mtime [db::getFilename $classname]] < [file mtime $wrpheader]} {
		return
	}

	set base [db::getParent $classname ]
	set wrpname Wrp_[set classname]
	set wrpscopename $wrpname
	set scopename $classname
	set wrpfileh [open $wrpheader w]

	set templatetype [db::getTemplate $classname]
	set isTemplate [string length $templatetype]

	if {$isTemplate} {
		set wrpfilecpp [open ../include/ivrs/gen/[string tolower $classname]wrp.tli w]
		regexp {template\s*(<.*?>)} $templatetype m T
		regsub -all {class|typename|unsigned|signed|bool|char|short|int|float|double|UINT} $T {} templateargs
		append scopename $templateargs
		append wrpscopename $templateargs
	} else {
		set wrpfilecpp [open ../src/gen/[string tolower $classname]wrp.cpp w]
	}

	set abstractClass [db::isAbstract $classname]
	set hasCtor       [db::hasCtor    $classname]
	set hasChilds     [db::hasChilds  $classname]
	set isColor       [expr [string equal $classname Color]]
	set isVector      [expr [string equal $classname Vector]]
	set isSO          [expr !$isColor && !$isVector && [expr [string compare $base ""] || [string equal $classname SharedObj]]]

	puts $wrpfilecpp "#include <ivrs/stringconvert.h>"
	foreach dep [db::getDependingClasses $classname ] {
		puts $wrpfilecpp "\#include <ivrs/gen/[string tolower $dep]wrp.h>"
	}

	puts $wrpfilecpp "#include <ivrs/methodtable.h>"
	puts $wrpfilecpp "#include <ivrs/wrappermanager.h>"
	if {[db::hasEnums $classname ]} {
		puts $wrpfilecpp "#include <ivrs/enumtable.h>"
	}

	puts $wrpfileh "#ifndef WRP_[string toupper $classname]WRP_H"
	puts $wrpfileh "#define WRP_[string toupper $classname]WRP_H"

	if {$base == ""} {
		puts $wrpfileh "#include <ivrs/wrapper.h>"
		set basewrpname Wrapper
	} else {
		regsub -all {<.*?>} $base {} basefile
		regsub -all {\s+|>|<} $basefile {} basefile
		puts $wrpfileh "#include <ivrs/gen/[string tolower $basefile]wrp.h>"
		set basewrpname Wrp_$base
	}

	puts $wrpfileh "#include \"../../[db::getFilename $classname ]\""

	puts $wrpfileh  "\nIVRS_NAMESPACE_BEGIN\nusing namespace VRS;\nusing namespace LDX;\n"
	puts $wrpfilecpp  "\nIVRS_NAMESPACE_BEGIN\nusing namespace VRS;\nusing namespace LDX;\n"

	if {$isTemplate} { puts $wrpfileh $templatetype }
	puts $wrpfileh "class $wrpname : public $basewrpname \{"
	puts $wrpfileh "public:"

	# special ctor and destroyObject for SharedObj
	if {$classname == "SharedObj"} {
		puts $wrpfileh "\t$wrpname\(void* obj = 0) : $basewrpname\(obj\) \{ if \(obj_\) \{ reinterpret_cast<SharedObj*>\(obj_\)->ref\(\); \} \}"
		puts $wrpfileh "\tvirtual void destroyObject\(\) \{  if \(obj_\) \{ reinterpret_cast<SharedObj*>\(obj_\)->unref\(\); \} obj_ = 0; \}"
	} else {
		puts $wrpfileh "\t$wrpname\(void* obj = 0) : $basewrpname\(obj\) \{ \}"    
		if {!$isSO} {
			puts $wrpfileh "\tvirtual void destroyObject\(\) \{  delete reinterpret_cast<$scopename*>\(obj_\); obj_ = 0;  \}"
		}
	}

	puts $wrpfileh "\tvirtual Wrapper* clone\(void* obj\) const \{ return new $wrpscopename\(obj\); \}"
	puts $wrpfileh "\tstatic WrapperClassInfo* WrapperInfo\(\);"
	puts $wrpfileh "\tvirtual WrapperClassInfo* wrapperInfo\(\) const \{ return $wrpscopename\::WrapperInfo\(\); \};"

	if {$isTemplate} { puts $wrpfilecpp $templatetype }
	puts $wrpfilecpp "WrapperClassInfo* $wrpscopename\::WrapperInfo\(\) \{"
	puts $wrpfilecpp "\tstatic WrapperClassInfo i\( typeid\($scopename\) , $basewrpname\::WrapperInfo\(\)\);"
	puts $wrpfilecpp "\treturn &i;\n\}"


	if {[db::hasEnums $classname ]} {
		puts $wrpfileh "\n\tvirtual EnumTable* enumtable\(\) const \{ return $wrpname\::Enumtable\(\); \}"
		puts $wrpfileh "\tstatic EnumTable* Enumtable\(\);"
	} else {
		puts $wrpfileh "\tvirtual EnumTable* enumtable\(\) const \{ return 0; \}"
	}

	set tablepart ""

	# make tcl interface for methods
	foreach tag [db::getMethodTags $classname ] {

		set name [db::getMethodName $classname $tag]
		set returnType [db::getMethodReturnType $classname $tag]
		set args [db::getMethodArgs $classname $tag]
		set flags [db::getMethodFlags $classname $tag]
		set isStatic [regexp {static} $flags]
		set isCtor [regexp {ctor} $flags]

		if {[db::overridesAbstractMethod $classname $tag]} {
			# Method implementation already done in base class
			continue
		}

		# template and unsigned hack backwards
		regsub -all {%} $returnType { } returnType

		# exclude methods
		if {($abstractClass && $isCtor) ||
			[regexp -expanded $excludeArgs "$args $returnType"] ||
			[regexp -expanded $excludeMethods $name]} {
			continue
		}

		# clean operators
		if {$isCtor} {
			set mname Ctor
		} else {
			set mname $name
			if {[regexp {operator} $mname]} {
				regsub {operator} $mname {} mname
				regsub -all {\[\]} $mname {get} mname
			}
		}

		# def " replaced by \"
		set defs [db::getMethodDefs $classname $tag]
		regsub -all {"} $defs {\"} defs

		# determine minimal and maximal number of arguments
		set min 0
		foreach def $defs { if {$def == ""} { incr min } }

		set mt METHOD
		if {$isStatic} { set mt STATIC }
		if {$isCtor} { set mt CTOR }
		append tablepart "\t\tdta_->insert\(\"$mname\",\(Wrapper::FKT\)&$wrpscopename\::$tag,\"$returnType\",\"$args\",\"[db::getMethodVars $classname $tag]\",\"$defs\",$min,[llength $args],MethodTable::$mt\);\n"

		# method signature
		puts $wrpfileh "\n\t// $returnType $name \( $args \)"
		puts $wrpfileh "\tconst char* $tag\(int argc, const char** argv);"

		puts $wrpfilecpp "\n// $returnType $name \( $args \)"
        if {$isTemplate} { puts $wrpfilecpp $templatetype }
		puts $wrpfilecpp "const char* $wrpscopename\::$tag\(int argc, const char** argv) {"

		# prefix for method call
		set prefix ""

		if {$isCtor} {
			# method call and argument conversion
			if {$isSO } {
				writeMethodSignatur $classname $wrpfilecpp "$scopename* myobj = new $scopename" $tag
				puts $wrpfilecpp "\tmyobj->ref\(\);"
				puts $wrpfilecpp "\tobj_ = myobj;"
			} else {
				writeMethodSignatur $classname $wrpfilecpp "obj_ = new $scopename" $tag
			}

		} else {
			# creating return type if != void
			if {[string compare $returnType void ] != 0} {
				if {$returnType != "ID"} {
					append prefix "const $returnType value = "
				} else {
					append prefix "const ID& value = "
				}
			}

			# class or object method
			if {$isStatic} {
				append prefix "$scopename\::$name"
			} else {
				append prefix "reinterpret_cast<$scopename*>\(obj_\)->$name"
			}

			# method call and argument conversion
			writeMethodSignatur $classname $wrpfilecpp $prefix $tag
		}

		# Conversion of return type and setting of TclResult string
		if {[string compare $returnType void ] != 0 && !$isCtor} {
			# care of OpenGL defines
			if {[regexp {GLenum|GLint} $returnType]} { set gl GL } else { set gl "" }

			# return to interpreter
			puts $wrpfilecpp "\treturn type2str$gl\(value\);"
		} else {
			puts $wrpfilecpp "\treturn 0;"
		}

		# method end
		puts $wrpfilecpp "}"
	}


	if {$tablepart != ""} {
		# method table
		puts $wrpfileh "\n\tvirtual MethodTable* methodTable\(\) const;"
		puts $wrpfilecpp ""
		if {$isTemplate} { puts $wrpfilecpp $templatetype }
		puts $wrpfilecpp "MethodTable* $wrpscopename\::methodTable\(\) const \{"
		puts $wrpfilecpp "\tstatic MethodTable* dta_ = 0;"
		puts $wrpfilecpp "\tif \(!dta_\) \{"
		puts $wrpfilecpp "\t\tdta_ = new MethodTable\($basewrpname\::methodTable\(\)\);"
		puts $wrpfilecpp "$tablepart"
		puts $wrpfilecpp "\t\}"
		puts $wrpfilecpp "\treturn dta_;"
		puts $wrpfilecpp "\}"
	}

	# close class declaration
	puts $wrpfileh "\};"

	# enums into enum manager
	if {[db::hasEnums $classname]} {
		puts $wrpfilecpp "EnumTable* $wrpname\::Enumtable\(\) \{"
		puts $wrpfilecpp "\tstatic EnumTable* etb_ = 0;"
		puts $wrpfilecpp "\tif\(!etb_\) \{"
		puts $wrpfilecpp "\t\tetb_  = new EnumTable\($wrpname\::WrapperInfo\(\)\);"
		foreach {enumname enumlist} [db::getEnums $classname] {
			puts $wrpfilecpp "\t\tstatic const char* str$enumname = \"$enumname\";"
			foreach e $enumlist {
				puts $wrpfilecpp "\t\tetb_->registerEnum\(\"$e\", str$enumname, $classname\::$e\);"
			}
		}
		puts $wrpfilecpp "\t\}"
		puts $wrpfilecpp "\treturn etb_;"
		puts $wrpfilecpp "\}"
	}

	# type2str + str2type for all enums in $classname
	foreach {enumtype enumlist} [db::getEnums $classname ] {
		puts $wrpfileh "const char* type2str\(const $classname\::$enumtype& e\);"
		puts $wrpfilecpp "const char* type2str\(const $classname\::$enumtype& e\) \{"
		puts $wrpfilecpp "\treturn $wrpname\::Enumtable\(\)->enumName\(e,\"$enumtype\"\);"
		puts $wrpfilecpp "\}"

		puts $wrpfileh "$classname\::$enumtype str2type\(const char* in, $classname\::$enumtype*\);"
		puts $wrpfilecpp "$classname\::$enumtype str2type\(const char* in, $classname\::$enumtype*\) \{"
		puts $wrpfilecpp "\treturn \($classname\::$enumtype\)\(\($wrpname\::Enumtable\(\)->enumValue\(in,\"$enumtype\"\)\)\);"
		puts $wrpfilecpp "\}"
	}

	if {!$isTemplate} {
		# writing initialisation part
		puts $wrpfilecpp "static PrototypeRegister instanceOf$wrpname\(new $wrpname\(0\)\);"
	}

	puts $wrpfileh  "\nIVRS_NAMESPACE_END\n"

	if {$isTemplate} { puts $wrpfileh "#include <ivrs/gen/[string tolower $classname]wrp.tli>" }

	puts $wrpfileh "\n\n\#endif //WRP_[string toupper $classname]WRP_H"
	puts $wrpfilecpp  "\nIVRS_NAMESPACE_END\n"

	# close source files
	close $wrpfileh
	close $wrpfilecpp
} errorvar

if {$errorvar != ""} { puts stderr $errorvar }

}


# erase files in 'gen' directory
if {!$OptimizeGenerator} {
	foreach f [glob -nocomplain ../include/ivrs/gen/* ../src/gen/*] {
		file delete $f
	}
}

# generate wrapper classes for all parsed classes
foreach classname [db::getClasses] {
	puts "+++ generating $classname"
	writeWrapperClass $classname
	update
}
